
/**
*
*   @define {HTML5GameContaier} This class will be used by all Games Portal (games.disney.com, play.disney.com) HTML5 games.
*   It will help simplify and unify general game portal processes like tracking, social network sharing,
*   and future planned features like rating games.
*
*
*
*   ==================================================================================
*   ==================================================================================
*   WINDOW (Global) level variables
*       
*       gameObj global level game container object
*   ==================================================================================
*   ==================================================================================
*
*   Change 2.1.9    Taking out Facebook
*   Change 2.1.8    Backwards compatibility proofing.
                    Corrected a problem with OpenFL adding moz-opaque to the canvas.  This would cause
*                   our canvas to be solid black and hide the game underneath.
*   Change 2.1.7.1  Made all console.log calls go to trace so I can turn them off in production
*   Change 2.1.7 -  Removed references to 'Score' to 'message' to make sharing more generic
*   Change 2.1.6 -  Generalizing more of the build process such as @SCORE@ -> @placeholder@
*   Change 2.1.5 -  Removed these changes
*   Change 2.1.3 -  An update to load sequence.
*   Change 2.1.2 -  Mouse and Touch are now being added via this class instead of on the index.html file.
*                    Events are now added via javascript and not jQuery.
*   Change 2.1.1 -  Mouse and Touch Events are now added to the parent DIV and passed down to
*                    all the canvases on the page.
*   Change 2.0.9 -  added more flexibility to the display of the sharing icons.
*                     • you can now change text size, text color, font adn actual verbiage of the 
*                        sharing caption
*                     • this script automatically measures the size of the sharing text caption so
*                        centering the icons under the text is easier.
*                     • you can choose which icons and in what order they appear.
*   Change 2.0.8 -  trying to implement facebook
*   Change 2.0.7 -  Added Text Color Attribute to the endGame method to be able to change the color
*                   of the 'Share Your Score!' text.
*/

/**
*   What version is this?
*/
var version = "2.1.8.1";
console.log("HTMLGameContainer loaded version "+version);
/**
*   Game link we want to use to track hits on it, and to include in Social Sharing messages.
*/
var vUrl = window.gameProps.gameLink,
/**
*   Just makes the global gameProps variable local
*/
gameProps = window.gameProps;

/*
*   CONSTANT PORTRAIT = 1  -- to test against whether or not a game is supposed to play in portrait mode or not.
*/
PORTRAIT = 1;


console.log("HTML5 Container Is window fired?  ["+window.loadFired+"]  containerLoaded ["+window.containerLoaded+"]");


var HTML5GameContainer = function (){
    // two step loading process for our games
     if(window.loadFired) {

        console.log("Page is completely loaded according to HTML5GameContainer.js");
        window.continue();
    }

    window.containerLoaded = true;
    
    // allows me to turn of console output for this file only
    var allowTrace = true;
    
    var hudContainer = "HUDcanvas",
        age,
        nextStep = function() {},
        minAge = 13,
        elem,
        active=false,
        left=0,
        top=0,
        message=0,
        parent,
        gameHUDLocation,
        name = "HTML5GameContainer", 
        x,y, canvasElem,
        elemLeft,
        elemTop ,
        context,
        elements = [],
        activeIcons= ['f','t','p'],
        _shareTextColor = "#FFFFFF",
        _shareCaption = "Share Your Score!",
        _shareTextSize = 24,
        _shareFont = "Arial",
        _shareIconSize=50,
        initialized=false,
        //  Check for cookie.  Is the user over 13  (true)?
        age = _checkCookie();
        /**
        *
        *   @define top int where the top of the icon appears in the sprite sheet
        *   @define left in where the left of the icon appears in the sprite sheet
        *   @define callBack string the function called when the icon is clicked
        *
        *   icons is an object (associate array) of properties we will use to load the 
        *   social network logos on screen when needed.
        *
        */
        icons = {'f':{top:152,left:10, callBack:'facebook', color:'#3C5A9A'}, 't':{top:152,left:130, callBack:'twitter', color:'#60A9DE'}, 'p':{top:152,left:70, callBack:'pinterest', color:'#CB2028'}};

    /**
    *   @define {function} Called when someone is rating a game.
    *
    *   @example window.ClickedRating(2);
    *
    *   @param rate The game rating that should be recorded/tracked.    
    *   
    */
    function clickedRating(rate){
        var args = {"action":"Rated", "message": rate};
        window.DIBIGameTracking( args );
    };

    
    /**
    *   @define {function} removes the age gate window.  
    *   @private
    */
    function _removeEndGameHUD(){
        // remove age gate
        publicAPI.closeAgeGate();
    };
    
    /**
    *   @define {function} Creates the canvas element that displays on top of the game.  It includes
    *   the age gate when necessary and all the social network icons that are requested for each game.
    *
    *   @private
    *   
    */
    function _initializeGameCanvas(){
        console.log("creating HUD canvas   parent ["+parent+"]");
        if(!initialized){
            
            
            // create the canvas tag that will be needed to display our icons
            $("<canvas id='"+hudContainer+"' name='"+hudContainer+"' class='hudCANVAS'></canvas>").insertAfter(parent); 

            // reference our new canvas element
            canvasElem = document.getElementById(hudContainer);

            _addCanvasAttributes();
            
            // get the context of the new canvas element
            context = canvasElem.getContext('2d'); 

            trace("Canvas element is ["+canvasElem+"]");

            initialized=true;

            parent.parentNode.addEventListener("mouseup", socialListener);
            parent.parentNode.addEventListener("touchend", touch);
            
             if(window.innerHeight > window.innerWidth){
                // Portrait orientation
                trace("Portrait  gameProps["+gameProps.orientation+"]");
                if(gameProps.orientation == PORTRAIT) publicAPI.goodRotation();
                else publicAPI.badRotation();
            } else {
                trace("Landscape gameProps["+gameProps.orientation+"]"); 
                if(gameProps.orientation != PORTRAIT) publicAPI.goodRotation();
                else publicAPI.badRotation();
            }
            
            window.addEventListener("resize", function() { 
                console.log("resize");
                _addCanvasAttributes(); 
                // what is this games orientation?
                if(window.innerHeight > window.innerWidth){
                    // Portrait orientation
                    trace("Portrait  gameProps["+gameProps.orientation+"]");
                    if(gameProps.orientation == PORTRAIT) publicAPI.goodRotation();
                    else publicAPI.badRotation();
                } else {
                    trace("Landscape gameProps["+gameProps.orientation+"]"); 
                    if(gameProps.orientation != PORTRAIT) publicAPI.goodRotation();
                    else publicAPI.badRotation();
                }
                
                drawIcons();
            });
        }
    }
    

    /**
    *   @define {function} touch event listener for mobile devices.
    *   @private
    *
    */
    function touch(event){
        trace("Called touch event");
        // find the size of the canvas
        var rect = canvasElem.getBoundingClientRect();
        
        // get the first touch event in the event
        var first = event.changedTouches[0];
        
        // find the x/y of the touch event which is offset by the elements offset attribute
        x = Math.round((first.pageX-rect.left)/(rect.right-rect.left)*canvasElem.width);
        y = Math.round((first.pageY-rect.top)/(rect.bottom-rect.top)*canvasElem.height);
        
        // find out of this coordinate is in our array of toucable elements.
        testHit(x,y);
    }
    
    
    /**
    *   @define {function} socialListener mouse event listener for desktops.
    *   @private
    *
    */
    function socialListener(event)
    {
            trace("Called SocialListener");
            var rect = canvasElem.getBoundingClientRect();

            x = Math.round((event.clientX-rect.left)/(rect.right-rect.left)*canvasElem.width);
            y = Math.round((event.clientY-rect.top)/(rect.bottom-rect.top)*canvasElem.height);
        
            testHit(x,y);
    }
    
    
    /**
    *   @define {function} testhit Iterate through our elements array and see if any of our registered
    *   elements are being hit.
    *
    *   @private
    *
    */
    function testHit(x, y){
        elements.forEach(function(element) {

            if (y > element.top && y < element.top + _shareIconSize 
                && x > element.left && x < element.left + _shareIconSize) 
            {
                element.callback();
            }

        });
    }
    
    
    function _addCanvasAttributes(){
        trace("adding canvas attributes    parent ["+parent.id+"]  parent style["+parent.style+"]");
        
        
        parent.style.position = "absolute";
        var attr=parent.attributes;
        // var st=parent.style;
        
        for(var i=0; i < attr.length; i++){
            trace("Styles ["+attr[i].name+"]  value ["+attr[i].value+"]");
            if(attr[i].name != "name" && attr[i].name != "id" && attr[i].name != "moz-opaque")  canvasElem.setAttribute(attr[i].name, attr[i].value);
        }
        
        // make sure the background stays transparent
        //canvasElem.setAttribute("background-color", "none");
        canvasElem.style.backgroundColor  = "transparent";
    }
    
    /**
    *   @define {function} draw place the icons on the screen. Called at endGame and on window.resize.
    *   @private
    */
    function drawIcons(){
        if(active){
            // elements array of screen elements that we need to redraw when ever the canvas needs to be 
            // redrawn.  Like after a resize window event or at first rendering of the social icons.
            elements = [];

            trace("draw canvasElem ["+canvasElem+"]  parent ["+parent.width+"] parent style["+parent.style+"]");

            _addCanvasAttributes();

            context.fillStyle = _shareTextColor;
            context.font = "bold "+_shareTextSize+"px " + _shareFont;
            context.fillText(_shareCaption,left,top);

            addIcons();
        }
    }
    
    /**
    *   @define addIcons {funciton} Display the icons the game is asking for
    *   @private
    */

    function addIcons()
    {
        // elements positioning 
        var vy = top + 10; // Icons should be 12 pixels below the text 
        var vx = left;
        var margin = 10;
        var textWidth = context.measureText(_shareCaption).width;
        // adjust the horizontal location of the icons to center under the text
        // 220 is the approx width of the text
        vx += (textWidth - ((_shareIconSize * activeIcons.length) + (margin * activeIcons.length-1))) /2;
        
        for(var i=0; i<activeIcons.length;i++){
            addIcon(activeIcons[i], vx + (i*(_shareIconSize+margin)), vy);
        }
    }
    
    /**
    *   
    *   @param which {string} which icon are we adding?
    *   @param whereX {int} what is the left screen coord where we want the icon to appear?
    */
    function addIcon(icon, whereX, whereY){
        trace("Adding ICON ["+icon+"]");
        var img = new Image();
        img.vx = whereX;
        img.vy = whereY;

        img.onload = function() {
            // trace("Loaded  img ["+img+"]  vx ["+this.vx+"]   vy["+this.vy+"]  left ["+icons[icon].left+"]  top ["+icons[icon].top+"]  _shareIconSize ["+_shareIconSize+"]");
            context.fillStyle = icons[icon].color;
            context.fillRect(this.vx,this.vy,_shareIconSize,_shareIconSize);
            var left = icons[icon].left
            var top = icons[icon].top
            context.drawImage(img, left, top,  50, 50, this.vx, this.vy, _shareIconSize, _shareIconSize);
        };

        img.src = '../gameContainer/img/social_modal_sprite.png';
        
       // img.addEventListener("click",function() { console.log ("Got the image click event") } , true);

        
        elements.push({
            top: img.vy,
            left: img.vx,
            //callback:_pinterestShare
            callback:function() { console.log("Clicked event"); publicAPI.share(icons[icon].callBack); }
        });
        
    }
    
    
    /**
    *   @define {function} _createAgeGateHUD Adds the window on top of the game and displays a drop down list to allow 
    *   the user to input their age.  We gate the user because we want to only allow 13+ yrs to share content
    *   on social networks
    *   @private
    */
    function _createAgeGateHUD(){
        trace("_createAgeGateHUD  parent["+parent+"]");
        // make sure the HUD does not already exist
        if(!$('#endgameHUD').length){
            $("<div id='endgameHUD'></div>").insertAfter(parent.parentNode);

            //$(parent.parentNode).click(function(){  console.log("Clicked on HUD"); this.click()});
            
            //var hudContainer = "#endgameHUD";
            var tempHud = document.getElementById('endgameHUD');
            // window.addEventListener("click", function(evt) { console.log("Dispatching "+ evt + " target ["+evt.target+"]"); window.dispatchEvent(evt); }, false);
            
            // clone the parents attributes        
            var attr=parent.attributes
            for(var i=0; i < attr.length; i++){
                if(attr[i].name != "name" && attr[i].name != "id") tempHud.setAttribute(attr[i].name, attr[i].value);
            }

            // make some necessary changes to the cloned attributes
            $('#endgameHUD').css("height", "100%");
            $('#endgameHUD').css("background-color", "transparent");
            $('#endgameHUD').css("display", "none");
            $('#endgameHUD').css("position","absolute");
            $('#endgameHUD').css("z-index","4444444");
            $('#endgameHUD').load("HUD.html", function() { HUDLoaded(); } );


            // document.body.onresize = function() { _adjustPosition(); } ;
        }

    }
    
    /**
    *
    *   @define {function} HUDLoaded once the HUD.html file is loaded we need to apply some CSS to it and create/add the age drop down.
    *   @private
    */
    function HUDLoaded(){
        trace("HUD LOADED");
        
        $('#hidden-div').css("display", "none");
        $("#endgameHUD").css("top", "10%");
        $("#outer-age-gate").css("display", "none");
        $("#outer-age-gate-sorry").css("display", "none");
        // add the datedropdown and the submit button to the page
        var dd = new DateDropDown("M:D:Y", $("#age-gate"));
        $('<center><button value="Submit" type="button" onclick="javascript:gameObj.submitAge();">Submit</button></center>').appendTo( $("#age-gate") );
    }

    
    /**
    *
    *   Here for future use
    *   @private
    */
    function redirect(){
        var str= "https://www.facebook.com/dialog/feed?";
        str += "app_id=145634995501895";
        str += "&display=popup&caption=Here is what I am sharing";
        str+= "&link=games.disney.com";
        str += "&redirect_uri=https://developers.facebook.com/tools/explorer";


        var myWindow = window.open("", "Share", "width=200, height=100");
        myWindow.location.href = str;
    };

    
    function _faceBookShare(){
        
         FB.ui({
            method: 'feed',
            // dialog:'my posting',
            name: gameProps.facebookCaption,
            link: gameProps.gameLink,
            href: gameProps.gameLink,
            picture: gameProps.facebookImg,
            caption: document.title,
            description: _changeVerbiage(gameProps.facebookVerbiage),
            message: 'Sharing My Score!'
            },  function(response) {
                if (response && response.post_id) {
                      alert('Post was published.');
                } else {
                    alert('Post was not published.');
                    }
                }
        );
        window.DIBIGameTracking({action:"Sharing to Facebook"});
    }
    
    function _twitterShare(){

        window.DIBIGameTracking({action:"Sharing to Twitter"});
        var myWindow = window.open("", "Share", "width=650, height=441");
        var vLink = "https://twitter.com/intent/tweet?url=" + vUrl+"&text="+_changeVerbiage(gameProps.twitterVerbiage);
        
        
        myWindow.location.href = vLink;
    }
    
    function _pinterestShare(){
        window.DIBIGameTracking({action:"Sharing to Pinterest"});
        var myWindow = window.open("", "Share", "width=650, height=441");
        var vLink = "http://www.pinterest.com/pin/create/button/?url="+vUrl+"&media="+gameProps.pinterestImg+"&amp;description="+_changeVerbiage(gameProps.pinterestVerbiage);
        
        myWindow.location.href = vLink;
    }
    
    function _changeVerbiage(verbiage){
        // score is the HTML5GameContainer 2.1.3 way of doing messaging.
        if(verbiage.indexOf("@SCORE@") > -1){
            var tempStr = verbiage.split("@SCORE@");
            return ( tempStr[0] + message + tempStr[1] );
        }
        // changing the message to have @placeholder@ instead because its more generic and 
        // will help not to confuse developers as to its meaning.
        if(verbiage.indexOf("@placeholder@") > -1){
            var tempStr = verbiage.split("@placeholder@");
            return ( tempStr[0] + message + tempStr[1] );
        }
    }
    
    // the user is too young to post to facebook.  Display a message to let them know.
    function _respondToAge(){
        trace("responding to age");
        $("#outer-age-gate").css("display", "none");
        $("#outer-age-gate-sorry").css("display", "block");
    }
    
    function _adjustPosition(){
        $(hudContainer).css("left",parent.style.left);
        $(hudContainer).css("width",parent.style.width);
    }


    function _ageGate(){
        _openAgeDiv();
    }

    function _openAgeDiv(){
        console.log("opening age div");
        $("#endgameHUD").css("display","block");
        $("#outer-age-gate").css("display", "block");
        $("#outer-age-gate-sorry").css("display", "none");
    };
    
    //  Store age cookie
    function _checkCookie(){
        var name = "age=";
        var ca = document.cookie.split(';');

        for(var i=0; i<ca.length; i++) {
            var c = ca[i];
                //name = i
            trace("Cookie ==> " + c + " is c=name ["+name+"]");
            if (c.indexOf(name) != -1) 
            {
                var temp = c.split("=");
                trace("age ["+temp[1]+"]");
                return temp[1];
            }
        }

        trace("CheckCookie == undefined");
        return undefined;
    };
    
        // Store True for user over 13
        function _storeCookie(cvalue){
            var d = new Date();
            var cname = "age"; // name of the cookie
            var exdays = 1; // 1 day expiration on the cookie
            d.setTime(d.getTime() + (exdays*24*60*60*1000));
            var expires = "expires="+d.toUTCString();
            document.cookie = cname + "=" + cvalue + "; " + expires;
        };
    
        /**
        *   Take Age info from Input Elements and check the age against our minimum age.
        *   @private
        */
       function _submitAge(){
            trace("Submitting age");
            // get todays date
            var date = new Date();
            
            // submitted age
            var age = new Date($("#Year").val(),$("#Month").val(),$("#Day").val(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds()); 
            // submit 13 years from today
            var testAge = new Date(date.getFullYear() - minAge, date.getMonth() , date.getDay(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds());
            // store it in coookie
            _storeCookie(age < testAge);
            // checkAge and respond
            if(age < testAge) {
                // close the age gate
                publicAPI.closeAgeGate();
                // call the social api that was requested
                nextStep();
            }
            else { _respondToAge(); }
        }
    
    
    function trace (msg) {
        if(allowTrace) console.log(msg);   
    }
    
    
    // ====================================================
    // ====================================================
    // ================  PUBLIC ACCESS  ===================
    // ====================================================
    
    var publicAPI = { 
        name:'HTML5GameContainer',
        
        /**
        *   @define {function} displayIconsFor - will store the supplied string to later know which icons to display.
        *   @param {string} icons - chars that represent which icons to display.  Item Delimiter is a : (colon).
        *       Used a delimiter incase we need more than single letter designations.
        *   @example displayIconsFor('f:t');  // will display the Facebook and Twitter logo icons.
        *
        *   As of version 2.0.8 possible letters are 'f','t','p'
        */
        displayIconsFor:function(icons){
            // convert icons into an array
            activeIcons = icons.split(":");
        },
        
        /**
        *   @define {function} shareIconSize - Set the size of the Social Network Icons when on screen.
        *   @param {int} size - pixel size of the font.
        */
        set shareIconSize(size) {
            trace("Setting shareIconSize Size to ["+size+"]");
            _shareIconSize = size
        },
        
        /**
        *   @define {function} shareTextSize - Set the text size of the Share Your Text caption.
        *   @param {int} size - pixel size of the font.
        */
        set shareTextSize(size) {
            trace("Setting shareText Size to ["+size+"]");
            _shareTextSize = size;
        },
        
        /**
        *   @define {function} shareFont - Set the font face of the share caption text.
        *   @param {string} fontFace.
        */
        set shareFont(fontFace) {
            _shareFont = fontFace;
        },
        
        /**
        *   @define {function} shareCaption Change the caption displayed above the sharing icons.
        *   @param {string} caption
        */
        set shareCaption (caption){
            _shareCaption = caption;   
        },
        
        /**
        *   @define {function} shareTextColor Set the color of the Share Your Text caption.
        *   @param {string} color Needs the # (#FFFFFF).
        */
        set shareTextColor(color){
            _shareTextColor = color;  
        },
        
        /**
        *   @define {function} setUpShareText - Set all the attributes of the share caption at once.
        *   @param {string} caption What do you want the text field to say
        *   @param {int} size - pixel size of the font.
        *   @param {string} color Needs the # (#FFFFFF).
        *   @param {string} fontFace.
        */
        setUpShareText:function ( caption , color , size, fontFace )
        {
            shareCaption(caption);
            shareTextColor(color);
            shareTextSize(size);
            shareFont(fontFace);
        },
        
        /**
        *
        *   @define {function} startGame Called when the start of a game is requested.  This will set up variables
        *   for use in later calls as well as clean up the screen of any other event screens that have loaded.
        *
        *   @depricated (@ version 2.0.9)
        *   This method is here to stay backward compatible with older games (Cinderella Midnight Dash only at this moment)
        */
        startGame:function() {
            trace("startGame in HTML5GameContainer");
            
            publicAPI.closeSharing();
        },
        
        /**
        *
        *   @depricated (@ version 2.0.9)
        *   @see displaySharing 
        *   This method is here to stay backward compatible with older games (Cinderella Midnight Dash only at this moment)
        *
        */
        endGame:function(parnt, gameHUD, scr, x, y) {
            // for backwards compatibility we need to initialize the canvas first incase it isn't
            publicAPI.initializeGameCanvas(parnt);
            // display the HUD
            publicAPI.displaySharing(scr, x, y);
        },
        /**
        *
        *   @depricated (@ version 2.0.9)
        *   @see displaySharing 
        *   This method is here to stay backward compatible with older games (Cinderella Midnight Dash only at this moment)
        *
        */
        displaySharing:function(){
            showSharing(scr,x,y);
        },
        
        /**
        *   @define - here to keep older games backward compatible
        *   @depricated
        */
        showScoreSharing:function(msg, x, y){
            publicAPI.showSharing(msg, x, y);
        },
        /**
        *
        *   @param scr String/Int - this is the end game score to share.
        *   @param x,y {string} Will set Top=y and Left=x.
        *   @param icons {string}  Display only these Social Network icons
        *   Settings and Methods that should always happen at the end of a game no matter what kind it is.
        *
        */
        showSharing:function(msg, x, y, icons){
            // convert icons into an array
            if(icons != undefined && icons != "") activeIcons = icons.split(":");
            message = msg;
            left = x;
            top = y;
            
            // display the Age Gate if necessary
            _createAgeGateHUD();
            active=true;
            drawIcons();
        },
        
        // to keep this file backward compatible
        closeHUD:function (){
            publicAPI.closeAgeGate();
        },
        
        closeSharing:function (){
             if(canvasElem != undefined){
                 active=false;
                // remove event listeners because we will be applying them again
                //canvasElem.removeEventListener('click', socialListener);
                //canvasElem.removeEventListener('touchend', touch, false);

                // document.body.onresize = null;
                _removeEndGameHUD();
                elements = [];
                //var canvas = document.getElementById(canvasElem);
                var context = canvasElem.getContext('2d');
                context.clearRect(0, 0, canvasElem.width, canvasElem.height);

                //canvasElem.remove();
            }
        },
        
        /**
        *   @define {fuinction} submitAge Called by the age selection drop down once age has been selected.
        */
        submitAge:function(){
            _submitAge();   
        },
        
        /**
        *
        *   @param canvas - the canvas (HTMLCanvasElement) we are going to copy
        *               canvas was - the id of the game canvas that we are going to clone the attributes of
        */
        initializeGameCanvas:function(canvas){
            // leaving legacy code of allowing canvas to be a string (name) of an element instead of the element itself.
            console.log (" Canvas type of ["+canvas.tagName+"]");
            if(canvas.tagName == "CANVAS"){
                console.log("Canvas was HTMLCanvasElement");
                parent = canvas;
            } else {
                // find the canvas on screen
                parent = window.document.getElementById(canvas);
            }
            // initialize our canvas
            if( !initialized ) _initializeGameCanvas();
        },
        
        /**
        *
        *   @param where string  where does the user want to share a post? Only use this if you want to bypass the
        *       fuller functionality of showSharing
        *   @see showScoreSharing
        *   @author Greg Burks
        *   @since v 1.0
        */
        share:function(where) {
            // Has age been checked before?  IF so what is it?
            age = _checkCookie();
            
            // get our URL so we can display the link to the user
            $("#fbLink").val( vUrl );
            
            // show the hidden div
            $("#endgameHUD").css("display","block");
            $('#hidden-div').css("display", "block");
            
            //  store the where function for later use       
            switch(where.toLowerCase()){
                case "facebook":
                    nextStep = _faceBookShare;
                    break;
                case "twitter":
                    nextStep = _twitterShare;
                    break;
                case "pinterest":
                    nextStep = _pinterestShare;
                    break;
            }
            
            ///////////////////////////////////////////////
            /////   FOR TESTING PURPOSES
            /////////////////////////////////////////////// 
            //      age = undefined;
            ///////////////////////////////////////////////
            /////////////////////////////////////////////// 
            
            if(age == undefined) 
            {
                _ageGate();
            } else {
                // if age was defined 
                if(age == "true"){
                    
                    this.closeAgeGate();
                    
                    nextStep();

                 } else {
                     // display that the user can't access this option
                     _respondToAge();
                 }
            }
        }, 
        
        closeAgeGate:function() {
            trace("closeAgeGate");
            $("#endgameHUD").css("display","none");
            $('#hidden-div').css("display", "none");
            //$('#endgameHUD').css("top", top);
            $("#outer-age-gate").css("display", "none");
            $("#outer-age-gate-sorry").css("display", "none");
           
        },
        // turn hud pieces back on after rotating to the proper orientation
        goodRotation:function(){
            $("#"+hudContainer).css("display", "block");
            $("#endgameHUD").css("display", "block");
        },
        // hide the hud while in the wrong orientation
        badRotation:function(){
            $("#"+hudContainer).css("display", "none");
            $("#endgameHUD").css("display", "none");
        }
    }
    
    return publicAPI;

};