define("screens/collectible", [
	"utils/Background",
	"sdk/EventManager",
	"sdk/DisplayObject",
	"sdk/TextField",
	"sdk/GameData",
	"sdk/Tween",
	"text/Font",
	"text/Gradient"
], 
function (Background, EventManager, DisplayObject, TextField, GameData, Tween, Font, Gradient){
	var screen = function(){
		var thisRef = this,
			// Local
			tokens = {
				1: { x: 102, y: 124 },
				2: { x: 102, y: 342 },
				3: { x: 102, y: 124 },
				4: { x: 102, y: 342 },
				5: { x: 102, y: 124 }
			},
			levels = {
				1: { x: 739, y: 37 },
				2: { x: 794, y: 37 },
				3: { x: 847, y: 37 }
			},
			// Fonts
			headerTitleFont = new Font("headerTitleFont"),
			headerTextFont = new Font("headerTextFont"),
			artistNameFont = new Font("artistNameFont"),
			artistDescriptionFont = new Font("artistDescriptionFont"),
			// Elements
			page1 = new DisplayObject("page1"),
			page2 = new DisplayObject("page2"),
			page3 = new DisplayObject("page3"),
			headerTitle = new TextField("headerTitle"),
			headerText = new TextField("headerText"),
			prevBtn = new DisplayObject("prev"),
			nextBtn = new DisplayObject("next"),
			closeBtn = new DisplayObject("close"),

			totalPage = 3,
			currentPage = 1
		;

		Background.set("media/images/backgrounds/collectionBg.jpg");

		// Fonts

		headerTitleFont.font = "30px NexaBold";
		headerTitleFont.fillStyle = "#FFF";
		headerTitleFont.textBaseline = "alphabetical";
		headerTitleFont.paddingTop = 30;
		headerTitleFont.textAlign = "left";

		headerTextFont.font = "20px NexaBold";
		headerTextFont.fillStyle = "#EAFF00";
		headerTextFont.textBaseline = "alphabetical";
		headerTextFont.paddingTop = 30;
		headerTextFont.textAlign = "right";

		artistNameFont.font = "18px NexaBold";
		artistNameFont.fillStyle = "#FFF";
		artistNameFont.textBaseline = "alphabetical";
		artistNameFont.paddingTop = 18;
		artistNameFont.textAlign = "left";

		artistDescriptionFont.font = "16px NexaBold";
		artistDescriptionFont.fillStyle = "#73AFC9";
		artistDescriptionFont.textBaseline = "alphabetical";
		artistDescriptionFont.paddingTop = 16;
		artistDescriptionFont.textAlign = "left";
		artistDescriptionFont.verticalSpacing = 5;
		artistDescriptionFont.textWrap = true;

		// Default
		
		this.addChild(page1);

		page2.alpha = 0;
		this.addChild(page2);

		page3.alpha = 0;
		this.addChild(page3);

		var pages = [page1, page2, page3];

		headerTitle.width = 420;
		headerTitle.height = 100;
		headerTitle.x = 111;
		headerTitle.y = 77;
		headerTitle.font = headerTitleFont;
		headerTitle.setStrokeText("CollectionTitle");
		this.addChild(headerTitle);

		headerText.width = 380;
		headerText.height = 100;
		headerText.x = 595;
		headerText.y = 74;
		headerText.font = headerTextFont;
		headerText.setStrokeText("CollectionText");
		this.addChild(headerText);

		var icon;
		var gold;

		var numberGradient = new Gradient(0, 0, 0, 20);
			numberGradient.addColorStop(0, "#c69057").addColorStop(0.5, "#f7d061").addColorStop(0.6, "#FFF").addColorStop(0.67, "#ffdc55").addColorStop(1, "#ffdc76");

		for(var key in tokens){
			if(typeof tokens[key] != "function"){
				var artistRow = new DisplayObject("artist" + key);
					artistRow.x = tokens[key].x;
					artistRow.y = tokens[key].y;

				gold = true;

				for(var i = 1; i < 4; i++){
					if(GameData.get("level" + i + "Token" + key, true) == "false"){
						gold = false;
					}
				}	

				switch(key){
					case "1":
					case "2":
						page1.addChild(artistRow);
					break;
					case "3":
					case "4":
						page2.addChild(artistRow);
					break;
					case "5":
						page3.addChild(artistRow);
					break;					
				}

				var artistThumb = new DisplayObject("artist" + key + "Thumb");
					

				if(gold){
					var goldBg = new DisplayObject("goldBg");
						goldBg.loadBitmap("media/images/screens/collection/goldBg.jpg");
						goldBg.x = -9;
						goldBg.y = 4;
						goldBg.height = 216;
					artistRow.addChild(goldBg);
					artistDescriptionFont.fillStyle = "#fff";
					artistNameFont.fillStyle = numberGradient.export();
					artistThumb.loadBitmap("media/images/artists/collectionToken" + key + "G.png");
				}else{
					artistNameFont.fillStyle = "#FFF";
					artistDescriptionFont.fillStyle = "#73AFC9";
					artistThumb.loadBitmap("media/images/artists/collectionToken" + key + ".png");
				}				
					artistThumb.x = 0;
					artistThumb.y = 0;
				artistRow.addChild(artistThumb);

				var artistName = new TextField("artistName" + key);
					artistName.width = 240;
					artistName.height = 100;
					artistName.x = 150;
					artistName.y = 5;
					artistName.font = artistNameFont;
					artistName.setStrokeText("Artist" + key + "Name");
				artistRow.addChild(artistName);

				var artistDescription = new TextField("artistDescription" + key);
					artistDescription.width = 600;
					artistDescription.height = 200;
					artistDescription.x = 150;
					artistDescription.y = 32;
					artistDescription.font = artistDescriptionFont;
					artistDescription.setStrokeText("Artist" + key + "Description");
				artistRow.addChild(artistDescription);

				for(var i = 1; i < 4; i++){
					icon = "";
					if(GameData.get("level" + i + "Token" + key, true) == "false"){
						icon = "grey";
					}
		
					var token = new DisplayObject("level" + i + "Token" + key);
						token.loadBitmap("media/images/screens/collection/" + i + icon +".png");
						token.x = levels[i].x;
						token.y = levels[i].y;
						artistRow.addChild(token);
				}				
			}
		}

		closeBtn.loadBitmap("media/images/screens/chooseLevel/panelClose.png");
		closeBtn.x = 993;
		closeBtn.y = 70;
		closeBtn.releasedState = function(){
			this.loadBitmap("media/images/screens/chooseLevel/panelClose.png");
		};
		closeBtn.pressedState = function(){
			this.loadBitmap("media/images/screens/chooseLevel/panelClose.png");
		};
		this.addChild(closeBtn);

		prevBtn.loadBitmap("media/images/buttons/btnPrev.png");
		prevBtn.x = 84;
		prevBtn.y = 488;
		prevBtn.alpha = 0;
		prevBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnPrev.png");
		};
		prevBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnPrevPressed.png");
		};
		this.addChild(prevBtn);

		nextBtn.loadBitmap("media/images/buttons/btnNext.png");
		nextBtn.x = 973;
		nextBtn.y = 488;
		nextBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnNext.png");
		};
		nextBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnNextPressed.png");
		};
		this.addChild(nextBtn);

		// Methods
		
		this.onload = function(){};	
		
		this.update = function(){};
		
		this.onunload = function(){};

		this.close = function(){
			EventManager.broadcast("gotoScreen", "chooseLevel");		
		};

		this.prev = function(){
			if(currentPage > 1){
				currentPage--;
				pages[currentPage-1].alpha = 1;
				pages[currentPage].alpha = 0;
				prevBtn.alpha = 1;
				nextBtn.alpha = 1;
			}

			if(currentPage == 1){
				prevBtn.alpha = 0;
				nextBtn.alpha = 1;
			}	
		};

		this.next = function(){
			if(currentPage < totalPage){
				currentPage++;
				pages[currentPage-1].alpha = 1;
				pages[currentPage-2].alpha = 0;
				prevBtn.alpha = 1;
				nextBtn.alpha = 1;
			}

			if(currentPage == totalPage){
				prevBtn.alpha = 1;
				nextBtn.alpha = 0;
			}
		};
		
		this.binds = {
			"close": thisRef.close,
			"prev": thisRef.prev,
			"next": thisRef.next
		};
	};
	
	return screen;
});