define("screens/gameplay", [
	"utils/Background",
	"sdk/Tween",
	"sdk/Tween2",
	"tweens/Easings",
	"sdk/EventManager",
	"sdk/DisplayObject",
	"sdkcustom/DisplayObjectFixed",
	"sdk/TextField",
	"text/Font",
	"text/Gradient",
	"sdk/GameData",
	"sdkcustom/KeyStates",
	"sdk/Sprite",
	"sdkcustom/SpriteFixed",
	"sprites/SpriteSheet",
	"sdkcustom/DynamicWidthTextField",
	"core/Buffer"
], 
function (Background, Animations, Tween, Easings, EventManager, DisplayObject, DisplayObjectFixed, TextField, Font, Gradient, GameData, KeyStates, Sprite, SpriteFixed, SpriteSheet, DynamicWidthTextField, Buffer) {
	
	var screen = function(){
		
		var thisRef = this;
		
		var screenHolder = new DisplayObjectFixed();
		this.addChild(screenHolder);
		
		
		
		function fetchAndShuffle(theList){
			var removed = theList.shift();
			var staticHalf = Math.floor(theList.length/2);
			var shuffleHalf = theList.length - staticHalf;
			var insertIndex = staticHalf+Math.round(Math.random()*shuffleHalf);
			theList.splice(insertIndex,0,removed);
			return removed;
		}
		function removeFromArray(theArray,theItem){
			theArray.splice(theArray.indexOf(theItem),1);
		}
		
		function rotNorm(val) {
			while (val < 0) val += Math.PI * 2;
			while (val > Math.PI * 2) val -= Math.PI * 2
			return val;
		}
		function rotGap(goal, pos) {
			goal = rotNorm(goal);
			pos = rotNorm(pos);
			
			var defVal = goal - pos;
			var altVal = goal + (Math.PI * 2 * (goal>pos?-1:1)) - pos;
			
			return ( Math.abs(defVal) < Math.abs(altVal) )?defVal:altVal;
		}
		function rotGapAng(goal, pos){
			return rotGap(goal/180*Math.PI,pos/180*Math.PI)*180/Math.PI;
		}
		
		var keyboardMode = ! window.GameContainer.browserInfo.touchDevice;
		var keyStates;
		
		
		var jump_new, jump_held;
		
		
		var logicHeartbeats;
		
		var maxHeartbeatsPerFrame = 50;//3;
		var logicHeartbeatsPerSecond = 50;
		
		var currentLevel = parseInt(GameData.get("selectedLevel"), 10);
		
		
		
		
		
		var tileSize = 55;
		var pixelsPerBeat = tileSize*4;
		
		
		var cameraLeadDistX = 200;
		var cameraLeadDistY = 400;
		
		var jumpSpeed = GameData.get('jumpSpeed')-0;
		var gravity = GameData.get('gravity')-0;
		var notesToIntensityUp = GameData.get('notesToIntensityUp')-0;
		var intensityMults = GameData.get('intensityMults');
		for(var i=0; i<intensityMults.length; i++) intensityMults[i] = intensityMults[i]-0;//convert from string
		var glowAmts = GameData.get('intensityBrightnessRanges');
		for(var i=0; i<glowAmts.length; i++) {
			//I'm honestly not sure if the string conversion is still required, but playing safe for now
			glowAmts[i][0] = glowAmts[i][0]-0;
			glowAmts[i][1] = glowAmts[i][1]-0;
		}
		
		var notePts = GameData.get('notePts')-0;
		var noteBonusPts = GameData.get('noteBonusPts')-0;
		var artistRingPts = GameData.get('artistRingPts')-0;
		var artistBallPts = GameData.get('artistBallPts')-0;

		var bounceStrenght = GameData.get("bounceStrenght");
		
		var curIntensityNotes = 0;
		
		
		var playerCanJump = true;
		
		
		var gameover = false;
		
		var cameraX = 0;
		var cameraY = -cameraLeadDistY;
		
		var playerX = 0;
		var playerY = -300;

		var curStep = 0;
		var stepGoal = 5;
		
		var onGround = true;
		var wasOnGround = true;
		var isHurting = false;
		var isTop = false;
		var topFailed = false;
		
		var playerPushY = 0;
		var distanceBeatOffset = 0;
		
		
		
		var invincibilityCountdown = 0;
		
		
		
		var lastSongTime = 0;
		
		var now;
		var curBeat = -1;
		var withinBeat = 0;
		var beatNum = -1;
		
		var isBeatChange = false;
		
		var lastBeat = -1;
		
		
		
		
		var artistChoices = JSON.parse(GameData.get("artistsToGet"+currentLevel,true));
		var curArtistChoice = (artistChoices[0] == undefined)?Math.floor((Math.random()*5)+1):artistChoices[0];
		GameData.set("tokenFound", 0);
		
		var bgDark = new DisplayObjectFixed();
		bgDark.loadBitmap('media/images/gameplay/layers/BG_dark_'+currentLevel+'.jpg');
		screenHolder.addChild(bgDark);
		
		var bgGlow = new DisplayObjectFixed();
		bgGlow.loadBitmap('media/images/gameplay/layers/BG_glow_'+currentLevel+'.jpg');
		screenHolder.addChild(bgGlow);
		bgGlow.alpha = 0;

		var preload = new DisplayObject("preload");
			preload.loadBitmap("media/images/gameplay/objects/pickupG.png");
			if(currentLevel > 3){
				preload.loadBitmap("media/images/gameplay/layers/BG_dark_bonus.jpg");
				preload.loadBitmap("media/images/gameplay/layers/BG_glow_bonus.jpg");
			}

		var numberGradient1 = new Gradient(0, 20, 0, 40);
			numberGradient1.addColorStop(0, "#fff").addColorStop(0.6, "#50d2F7").addColorStop(1, "#4D90EA");

		var numberGradient2 = new Gradient(0, 20, 0, 40);
			numberGradient2.addColorStop(0, "#fff").addColorStop(0.6, "#91FF5B").addColorStop(1, "#359A0D");

		var numberGradient3 = new Gradient(0, 0, 0, 40);
			numberGradient3.addColorStop(0, "#DED22d").addColorStop(0.5, "#ffdc55").addColorStop(0.75, "#FFF").addColorStop(0.9, "#ffdc55").addColorStop(1, "#d09c57");

		var calloutGradient = new Gradient(0, 0, 0, 30);
			calloutGradient.addColorStop(0, "#c69057").addColorStop(0.4, "#f7d061").addColorStop(0.6, "#FFF").addColorStop(0.67, "#ffdc55").addColorStop(1, "#ffdc76");
		
		var scoreGradient = new Gradient(0, 10, 0, 50);
			scoreGradient.addColorStop(0, "#c69057").addColorStop(0.4, "#f7d061").addColorStop(0.6, "#FFF").addColorStop(0.67, "#ffdc55").addColorStop(1, "#ffdc76");

		
		
		var scoreLabelFont = new Font("scoreLabelFont");
		scoreLabelFont.font = "bold 26px GeomGraphicSemiBoldItalic";
		scoreLabelFont.fillStyle = '#f7de82';
		scoreLabelFont.textAlign = 'right';
		scoreLabelFont.textBaseline = "top";
		
		var scoreDispFont = new Font("scoreDispFont");
		scoreDispFont.font = "bold 44px GeomGraphicSemiBoldItalic";
		scoreDispFont.fillStyle = "#fff"/*scoreGradient.export()*/;
		scoreDispFont.textAlign = 'right';
		scoreDispFont.paddingTop = 51;
		scoreDispFont.paddingLeft = -5;
		scoreDispFont.textBaseline = "alphabetical";
		scoreDispFont.shadowColor = "black";
		scoreDispFont.shadowOffsetX = 2;
		scoreDispFont.shadowOffsetY = 2;
		
		var calloutFont = new Font("calloutFont");
		calloutFont.font = "bold 51px NexaBold";
		calloutFont.fillStyle = calloutGradient.export();
		calloutFont.textAlign = 'center';
		calloutFont.textBaseline = "top";
		/*calloutFont.shadowColor = "black";
		calloutFont.shadowOffsetX = 2;
		calloutFont.shadowOffsetY = 2;*/
		
		var scoreHolder = new DisplayObjectFixed("scoreHolder");
		scoreHolder.x = 1136 - 35;
		scoreHolder.y = 35;
		scoreHolder.scaleX = 0.8;
		screenHolder.addChild(scoreHolder);

		
		/*var scoreLabel = new TextField();
		scoreLabel.font = scoreLabelFont;
		scoreLabel.width = 200;
		scoreLabel.height = 100;
		scoreLabel.x = -scoreLabel.width;
		scoreLabel.setFillText('score');
		//scoreHolder.addChild(scoreLabel);*/
		
		var scoreDisp = new TextField("scoreDisp");
		scoreDisp.font = scoreDispFont;
		scoreDisp.width = 200;
		scoreDisp.height = 100;
		scoreDisp.x = -scoreDisp.width;
		scoreDisp.setFillText('0');
		scoreHolder.addChild(scoreDisp);
		
		
		var score = 0;
		var pickupTotal = 0;
		var notesCollected = 0;
		function addScore(pts) {
			score += pts;
			scoreDisp.killBuffer();
			scoreDisp.setFillText(score);
		}
		
		
		
		var bgArtist = new DisplayObjectFixed();
		bgArtist.loadBitmap('media/images/gameplay/layers/artist_'+curArtistChoice+((currentLevel <= 3)?"":"_2015")+".png");
		screenHolder.addChild(bgArtist);
		bgArtist.alpha = 0;
		
		var sunrayLayer = new DisplayObjectFixed();
		screenHolder.addChild(sunrayLayer);
		
		//Tutorial loading
		if(currentLevel == 0){
			var tutDist = GameData.get("tutDist");
			var tuts = [{"image":"media/images/gameplay/ui/tutorial_jump.png"},
						{"image":"media/images/gameplay/ui/tutorial_collect.png"},
						{"image":"media/images/gameplay/ui/tutorial_hurt.png"},
						{"image":"media/images/gameplay/ui/tutorial_token.png"}
						];
			var tutLayer = new DisplayObjectFixed();
			var currentTut = 0;
			screenHolder.addChild(tutLayer);
			for(var tut = 0; tut < tuts.length; tut++){
				var tutorial = new DisplayObjectFixed();
				if(window.GameContainer.browserInfo.touchDevice && tut == 0){
					tutorial.loadBitmap("media/images/gameplay/ui/tutorial_jump_mobile.png");
				}else{
					tutorial.loadBitmap(tuts[tut].image);
				}
				tutorial.alpha = 0;
				tutorial.x = window.GameContainer.visibleWidth/2 - 150;
				tutorial.y = 140;
				tutLayer.addChild(tutorial);
			}
		}
		
		
		var starMaxX = 93;
		var starMaxY = 57;
		var starGapX = 1121/(starMaxX-1);
		var starGapY = 656/(starMaxY-1);
		var starOffX = 8.5;
		var starOffY = 8.5;
		var starsX = 5;
		var starsY = 3;
		
		var starsActive = false;
		
		var bgStarsHolder = new DisplayObjectFixed();
		bgStarsHolder.alpha = 0;
		screenHolder.addChild(bgStarsHolder);
		var bgStarsList = [];
		for(var x=0; x<starsX; x++) {
			var curCol = [];
			for(var y=0; y<starsY; y++){
				var newStar = new DisplayObjectFixed();
				newStar.loadBitmap('media/images/gameplay/layers/star.png');
				newStar.baseStarX = Math.round((x+0.5)/starsX*starMaxX);
				newStar.baseStarY = Math.round((y+0.5)/starsY*starMaxY);
				bgStarsHolder.addChild(newStar);
				curCol.push(newStar);
			}
			bgStarsList.push(curCol);
		}
		
		function scrambleStars() {
			for(var x=0; x<bgStarsList.length; x++) {
				var curCol = bgStarsList[x];
				for(var y=0; y<curCol.length; y++){
					var curStar = curCol[y];
					curStar.x = (curStar.baseStarX+Math.round((Math.random()-0.5)*(starMaxX/starsX)))*starGapX + starOffX - 39;
					curStar.y = (curStar.baseStarY+Math.round((Math.random()-0.5)*(starMaxY/starsY)))*starGapY + starOffY - 35;
				}
			}
		}
		
		var world = new DisplayObjectFixed();
		screenHolder.addChild(world);
		
		var bonusGlow = new DisplayObjectFixed();
		bonusGlow.loadBitmap('media/images/gameplay/layers/bonus_overlay.png');
		screenHolder.addChild(bonusGlow);
		bonusGlow.alpha = 0;
		
		
		var hud = new DisplayObjectFixed();
		screenHolder.addChild(hud);
		
		var comboShine = new DisplayObjectFixed();
		hud.addChild(comboShine);
		//comboShine.y = 105;
		comboShine.alpha = 0;
		
		var comboShineInner = new DisplayObjectFixed();
		comboShineInner.loadBitmap('media/images/gameplay/ui/combosparkle.png');
		comboShineInner.x = -180;
		comboShineInner.y = -160;
		comboShine.addChild(comboShineInner);
		
		
		var intensityShineFrac = 1;
		var intensityShakeFrac = 1;
		
		var multLevel = 1;
		
		
		var comboHolder = new DisplayObjectFixed();
		comboHolder.x = 568 + 70;
		comboHolder.y = 130;
		comboHolder.scaleX = 0.7;
		comboHolder.scaleY = 0.85;
		comboHolder.skewX = -7/29;
		if(currentLevel != 0){
			hud.addChild(comboHolder);
		}else{
			var htpLabelFont = new Font("htpLabelFont");
				htpLabelFont.font = "bold 37px GeomGraphicSemiBoldItalic";
				htpLabelFont.fillStyle = '#000000';
				htpLabelFont.strokeStyle = '#F8EE7A';
				htpLabelFont.lineWidth = 5;
				htpLabelFont.paddingLeft = -5;
				htpLabelFont.lineJoin = "round";
				htpLabelFont.textAlign = 'center';
				htpLabelFont.textBaseline = "bottom";
				htpLabelFont.paddingTop = 54;

			var htpLabel = new TextField("htpLabel");
				htpLabel.font = htpLabelFont;
				htpLabel.width = 600;
				htpLabel.height = 60;
				htpLabel.x = 568 - htpLabel.width*0.5;
				htpLabel.y = 80;
				htpLabel.setStrokeText('HtpTitle');
				this.addChild(htpLabel);
		}
		
		var comboLabelFont = new Font("comboLabelFont");
			comboLabelFont.font = "bold 40px GeomGraphicSemiBoldItalic";
			comboLabelFont.fillStyle = '#fff';
			comboLabelFont.lineWidth = 5;
			comboLabelFont.paddingLeft = -5;
			comboLabelFont.lineJoin = "round";
			comboLabelFont.textAlign = 'right';
			comboLabelFont.textBaseline = "alphabetical";
			comboLabelFont.paddingTop = 54;
			comboLabelFont.shadowColor = "rgba(255, 255, 255, 1)";
			comboLabelFont.shadowColor = "black";
			comboLabelFont.shadowOffsetX = 3;
			comboLabelFont.shadowOffsetY = 3;
		//console.log(comboLabelFont);
		
		
		var comboNumberFont = new Font("comboNumberFont");
			comboNumberFont.font = "bold 50px GeomGraphicSemiBoldItalic";
			comboNumberFont.fillStyle = numberGradient1.export();
			comboNumberFont.strokeStyle = '#93ddff';
			comboNumberFont.lineWidth = 4;
			comboNumberFont.lineJoin = "round";
			comboNumberFont.paddingLeft = 5;
			comboNumberFont.textAlign = 'left';
			comboNumberFont.textBaseline = "alphabetical";
			comboNumberFont.paddingTop = 54;
			comboNumberFont.shadowColor = "rgba(255, 255, 221, 1)";
			comboNumberFont.shadowColor = "black";
			comboNumberFont.shadowOffsetX = 3;
			comboNumberFont.shadowOffsetY = 3;
		
		var comboLabel = new TextField("comboLabel");
			comboLabel.font = comboLabelFont;
			comboLabel.width = 200;
			comboLabel.height = 60;
			comboLabel.x = -comboLabel.width;
			comboLabel.y = -comboLabel.height-15;
			comboLabel.setFillText('Combo');
			comboHolder.addChild(comboLabel);

		var comboNumber = new TextField("comboNumber");
			comboNumber.font = comboNumberFont;
			comboNumber.width = 200;
			comboNumber.height = 60;
			comboNumber.x = 15;
			comboNumber.y = -comboNumber.height-12;
			comboNumber.setStrokeText('x',{"%%":1});
			comboHolder.addChild(comboNumber);
		//console.log(comboNumber);
		
		var tokenCallout = new TextField("tokenCallout");
			tokenCallout.font = calloutFont;
			tokenCallout.width = 400;
			tokenCallout.height = 100;
			tokenCallout.textAlign = "center";
			tokenCallout.x = window.GameContainer.visibleWidth/2 - 550;
			if(currentLevel == 0){
				tokenCallout.y = 300;
			}else{
				tokenCallout.y = 140;
			}
			tokenCallout.alpha = 0;
			tokenCallout.setFillText('1/4');
		hud.addChild(tokenCallout);
		
		
		var platformHolder = new DisplayObjectFixed();
		world.addChild(platformHolder);
		
		
		var nextTileIndex = 0;
		var curTileY = 0;
		var curTileBonusY = -12;
		
		var groundC = 0;
		
		
		
		
		
		
		
		
		var animators = [];
		function Animator(spriteSheetPath,offX,offY,spriteWidth,spriteHeight){
			this.spriteSheet = new SpriteSheet(spriteSheetPath);
			this.sprite = new SpriteFixed();
			this.sprite.x = -offX;
			this.sprite.y = -offY;
			this.sprite.width = spriteWidth;
			this.sprite.height = spriteHeight;
			this.holderDO = new DisplayObjectFixed();
			this.holderDO.addChild(this.sprite);
			this.animDone = true;//so it doesn't run before an animation has been loaded
			animators.push(this);
		}
		Animator.prototype.loadAnim = function(curAnim){
			//console.log("LOADING NEW ANIM");
			this.oneshot = curAnim.oneshot;
			this.curFrames = curAnim.frames;
			this.maxInterframes = curAnim.interframe;
			this.animDoneCallback = curAnim.callbackFunc;
			this.animFrame = -1;
			this.animInterframe = 0;
			this.animDone = false;
		}
		Animator.prototype.update = function(){
			
			if (this.animDone) return;
			
			this.animInterframe--;
			if (this.animInterframe < 0) {
				this.animFrame++;
				if (this.animFrame == this.curFrames.length) {
					
					if (this.animDoneCallback) this.animDoneCallback();
					
					if ( (this.animFrame == this.curFrames.length) && this.oneshot ) {//if it is STILL the case, i.e. if the callback has NOT reset the animation:
						this.animDone = true;
						return;
					} else {//whether it's been reset or it's not a oneshot, rewind to the first frame:
						this.animFrame = 0;
					}
					
				}
				this.animInterframe = this.maxInterframes;
				this.sprite.showFrame(this.spriteSheet,this.curFrames[this.animFrame]);
			}
			
		}
		Animator.prototype.kill = function(){
			removeFromArray(animators,this);
		}
		
		
		
		
		
		
		var runAnim = {frames:[0,1,2,3,4,5,6,7],interframe:3};
		var jumpAnim = {frames:[10,11],interframe:3,oneshot:true};
		var fallAnim = {frames:[12,13],interframe:3,oneshot:true};
		var landAnim = {frames:[14],interframe:3,oneshot:true,callbackFunc:function(){
			playerAnim.loadAnim(runAnim);
		}};
		var hurtAnim = {frames:[15,16,17,17,18,19,20],interframe:3,oneshot:true,callbackFunc:function(){
			isHurting = false;
			if (onGround) {
				playerAnim.loadAnim(runAnim);
			} else {
				if (playerPushY < 0) {
					playerAnim.loadAnim(jumpAnim);
				} else {
					playerAnim.loadAnim(fallAnim);
				}
			}
		}};
		
		var playerAnim = new Animator('media/images/gameplay/objects/Ardy_Anim.png',60,158,123,163);
		playerAnim.loadAnim(runAnim);
		world.addChild(playerAnim.holderDO);
		
		
		var playerColWr = 22;
		var playerColH = 120;
		
		
		
		
		var rayOffset = 1/4;
		var oldTwoBeat;
		var twoBeat;
		var twoBeatChange;
		var withinTwobeat;
		var curvedTwobeat;
		
		var raysActive = false;
		
		
		var sunrays = [];
		function SunRay(centerX,centerY,angStart,angEnd) {
			var innerDO = new DisplayObjectFixed();
				innerDO.loadBitmap('media/images/gameplay/layers/sunray.png');
				innerDO.y = -300;
			this.myDO = new DisplayObjectFixed();
			this.myDO.x = centerX;
			this.myDO.y = centerY;
			this.myDO.addChild(innerDO);
			sunrayLayer.addChild(this.myDO);

			this.angStart = angStart;
			this.angGap = rotGapAng(angEnd,angStart);
			this.randomizeBeam();
			sunrays.push(this);
		}
		SunRay.prototype.randomizeBeam = function(){
			this.curStartAng = this.angStart + Math.random()*this.angGap;
			var curEndAng = this.angStart + Math.random()*this.angGap;
			this.curGapAng = rotGapAng(curEndAng,this.curStartAng);
			this.myDO.scaleY = 0.5 - Math.random()*0.4;
		}
		SunRay.prototype.update = function(){
			this.myDO.rotation = this.curStartAng + this.curGapAng * curvedTwobeat;
		}
		
		
		for(var x=0; x<2; x++){
			for(var i=0; i<3; i++){
				new SunRay( x==0 ? -100 : 1236 , -100 , x==0 ? 0 : 180 , 90 );
			}
		}
		
		
		
		
		var cornerLockers = [];
		function addCornerLocker(cornerDO,xSide,ySide,xOff,yOff){
			cornerLockers.push({cornerDO:cornerDO,xSide:xSide,ySide:ySide,xOff:xOff,yOff:yOff});
		}
		if(htpLabel){
			addCornerLocker(htpLabel,0,-1,-280,15);
		}
		addCornerLocker(comboShine,0,-1,-60,55);
		addCornerLocker(comboHolder,0,-1,70,90);
		addCornerLocker(scoreHolder,1,-1,-25,10);
		
		var notesOnScreen = [];
		var activeEntities = [];
		function EntityFactory(args){
			
			var path = args.path;
			var buffer = args.buffer;
			var offsetX = args.offsetX ? args.offsetX : 0 ;
			var offsetY = args.offsetY ? args.offsetY : 0 ;
			var colWR = args.colWR ? args.colWR : 0;
			var colWL = args.colWL ? args.colWL : 0;
			var colHD = args.colHD ? args.colHD : 0;
			var colHU = args.colHU ? args.colHU : 0;
			var initFunc    = args.initFunc;
			var updateFunc  = args.updateFunc;
			var drawFunc    = args.drawFunc;
			var landFunc    = args.landFunc;
			var collideFunc = args.collideFunc;
			var missedFunc  = args.missedFunc;
			var onKillFunc  = args.onKillFunc;
			var holder = args.holder;
			var autoRemove = args.autoRemove;
			var forcePreciseJump = args.forcePreciseJump;
			var spriteInfo = args.spriteInfo;
			
			var extraDisposeW = args.extraDisposeW ? args.extraDisposeW : 0 ;
			
			
			
			if (path) {
				var entityLoader = new DisplayObjectFixed();
				entityLoader.loadBitmap('media/images/gameplay/objects/'+path+'.png');
			}
			
			
			
			this.offsetX = offsetX;// for looking up where it's safe to spawn offscreen
			
			this.spawn = function(x,y,initArgs){
				
				var newEntity;
				if (spriteInfo) {
					var theEntityAnimator = new Animator(entityLoader.data,0,0,spriteInfo.width,spriteInfo.height);
					newEntity = theEntityAnimator.holderDO;
					if(path == "goldtrail"){
						newEntity.children[0].rpX = 0.5;
						newEntity.children[0].rpY = 0.5;
					}
					newEntity.entityAnimator = theEntityAnimator;
					newEntity.entityAnimator.loadAnim(spriteInfo.initAnim);
					// for auto-remove
					newEntity.width = spriteInfo.width;
				} else {
					newEntity = new DisplayObjectFixed();
					if (buffer) newEntity.loadBitmap(buffer);
					if(path == "pickup" && multLevel == 4){
						if (path) newEntity.loadBitmap('media/images/gameplay/objects/pickupG.png');
					}else{
						if (path) newEntity.loadBitmap(entityLoader.data);
					}

				}
				newEntity.x = x-offsetX;
				newEntity.y = y-offsetY;
				
				newEntity.offsetX = offsetX;
				newEntity.offsetY = offsetY;
				newEntity.colWR = colWR;
				newEntity.colWL = colWL;
				newEntity.colHD = colHD;
				newEntity.colHU = colHU;
				newEntity.updateFunc = updateFunc;
				newEntity.drawFunc = drawFunc;
				newEntity.landFunc = landFunc;
				newEntity.collideFunc = collideFunc;
				newEntity.missedFunc = missedFunc;
				newEntity.autoRemove = autoRemove;
				newEntity.forcePreciseJump = forcePreciseJump;
				newEntity.extraDisposeW = extraDisposeW;
				
				newEntity.debugPath = path;
				if(path == "pickup"){
					pickupTotal++;
					notesOnScreen.push(newEntity);
				}
				activeEntities.push(newEntity);
				holder.addChild(newEntity);
				
				newEntity.kill = function(){
					if (spriteInfo) this.entityAnimator.kill();
					if (onKillFunc) onKillFunc.call(this);
					removeFromArray(activeEntities,this);
					holder.removeChild(this);
				}
				
				newEntity.getCenter = function(){
					return {
						x: this.x + this.offsetX - this.colWL + (this.colWR+this.colWL)/2,
						y: this.y + this.offsetY - this.colHU + (this.colHD+this.colHU)/2
					};
				}
				
				if (initFunc) initFunc.call(newEntity,initArgs);
				return newEntity;
				
			}
			
		}
		
		function platformLandFunc(){
			onGround = true;
			if (!isHurting && !wasOnGround) playerAnim.loadAnim(landAnim);
			playerY = this.y+this.offsetY;
		}
		
		function obstacleCollideFunc(){
			playerHit();
			if(invincibilityCountdown == 0){
				invincibilityCountdown = 100;
			}
		}

		function platformBonusLandFunc(){
			if(topFailed){
				isTop = false;
				return;
			}else{
				onGround = true;
				if (!isHurting && !wasOnGround) playerAnim.loadAnim(landAnim);
				playerY = this.y+this.offsetY;
			}
		}
		
		function obstacleBonusCollideFunc(){
			if(topFailed){
				return;
			}else{
				playerHit();
				if(invincibilityCountdown == 0){
					invincibilityCountdown = 100;
				}
			}
		}
		
		
		
		
		
		var END_WIN = 1;
		var END_FAIL = 2;
		var END_QUIT = 3;
		function endLevel(toWhere){
			
			for(var i=0; i<allMusicTracks.length; i++) allMusicTracks[i].pause();
			gameover = true;
			
			switch(toWhere){
				case END_WIN:
					GameData.set('scoreTotal', score); 
					GameData.set('notePercentage',Math.floor(notesCollected/pickupTotal*100));
					
					if(Math.floor(notesCollected/pickupTotal*100) > GameData.get('level'+currentLevel+"Note",true)){
						GameData.set('level'+currentLevel+"Note", Math.floor(notesCollected/pickupTotal*100),true);
					}

					if(GameData.get("stars")[currentLevel-1] && score >= GameData.get("stars")[currentLevel-1][0]){
						GameData.set('starTotal', 1);
						if(score >= GameData.get("stars")[currentLevel-1][1]){
							GameData.set('starTotal', 2);
							if(score >= GameData.get("stars")[currentLevel-1][2]){
								GameData.set('starTotal', 3);
							}
						}
					}else{
						GameData.set('starTotal', 0);
					}

					if(parseInt(GameData.get("starTotal"), 10) > parseInt(GameData.get("level" + currentLevel + "Star", true), 10)){
						GameData.set("level" + currentLevel + "Star", GameData.get("starTotal"), true);
					}


					if(parseInt(GameData.get("tokenFound"), 10) > 0){
						GameData.set("level" + currentLevel + "Token" + GameData.get("tokenFound"), "true", true);
						//console.log(JSON.parse(GameData.get("artistsToGet"+currentLevel,true)));
						artistChoices.shift()
						GameData.set("artistsToGet"+currentLevel,JSON.stringify(artistChoices),true);
					}

					if(parseInt(GameData.get("scoreTotal"), 10) > parseInt(GameData.get("level" + currentLevel + "Highscore", true), 10)){
						GameData.set("level" + currentLevel + "Highscore", GameData.get("scoreTotal"), true);
					}
					
					if(currentLevel == parseInt(GameData.get("levelUnlocked", true))){
						currentLevel++;
						GameData.set("levelUnlocked", currentLevel, true);
					}

					if(currentLevel<=3){
						EventManager.broadcast("showPopup", "popupWin");
					}else{
						EventManager.broadcast("showPopup", "popupWin2015");
					}
				break;
				case END_FAIL:	
				case END_QUIT:
					setTimeout(function(){
						Background.set(thisRef.engine.canvas.toDataURL("image/png"));
						EventManager.broadcast("showPopup", "popupFail");
					},0);
				break;
			}
			
		}
		
		
		
		
		var pickupFlashLoader = new DisplayObjectFixed();
			pickupFlashLoader.loadBitmap('media/images/gameplay/objects/pickup_flash.png');
		
		
		
		function pickupFlashInit(){
			var flashInner = new DisplayObjectFixed();
				flashInner.loadBitmap(pickupFlashLoader.data);
				flashInner.x = -64;
				flashInner.y = -64;
			this.addChild(flashInner);
			this.myBeat = this.x/pixelsPerBeat;
		}
		function pickupFlashUpdate(){
			this.flashAmt = curBeat - this.myBeat;
			this.scaleY = this.scaleX = Math.pow(1-Math.abs(this.flashAmt),2);
			if (this.flashAmt >= 1) this.kill();
		}
		
		
		
		function pickupCollectFunc(){
			var pickupCenter = this.getCenter();
			pickupFlashType.spawn(pickupCenter.x,pickupCenter.y);		
			this.kill();
		}
		function lifeCollectFunc(){
			lives++;
			lives = Math.min(lives,maxLives);
			updateLivesDisp();
			pickupCollectFunc.call(this);
			
			sfxSprite.play('Sfx_life_pick_up');
		}
		function invincibilityCollideFunc(){
			invincibilityCountdown = 200;
			pickupCollectFunc.call(this);
		}
		function bounceCollideFunc(){
			playerPushY = -bounceStrenght;
			isTop = true;
			
			bgDark.loadBitmap('media/images/gameplay/layers/BG_dark_bonus.jpg');
			bgGlow.loadBitmap('media/images/gameplay/layers/BG_glow_bonus.jpg');

			bonusGlow.alpha = 1;

			if(GameData.get("muted", true) != "true"){
				allMusicTracks[0].fade(1,0,100);
				allMusicTracks[1].fade(0,1,100);
			}
		}
		function bonusEndCollideFunc(){
			bonusEnded();
		}
		var comboShineOffX = 0;
		function artistCollectFunc(){
			collectedBalls++;
			tokenCallout.killBuffer();
			if (collectedBalls == totalBalls) {
				//artist unlock logic
				addScore(artistBallPts);
				tokenCallout.setFillText("AllToken");
				GameData.set("tokenFound", curArtistChoice);
				if(currentLevel != 0){
					tokenCallout.tween = new Tween(tokenCallout, {x:window.GameContainer.visibleWidth/2  - 150, alpha:1}, 800, Easings.QUAD_IN_OUT).then({x:window.GameContainer.visibleWidth/2 - 150 + 20}, 1300, Easings.LINEAR).then({x:window.GameContainer.visibleWidth/2 + 500, alpha:0}, 500, Easings.QUAD_IN_OUT).then(function(){
					tokenCallout.x = window.GameContainer.visibleWidth/2 - 400;
					tokenCallout.killBuffer();
					tokenCallout.setFillText("Artist"+curArtistChoice+((currentLevel <= 3)?"Name":"Name_2015"));
					comboShine.y += 75;
					comboShineOffX = -90;
					intensityShineFrac = 0;//start
					intensityShakeFrac = 0.99;//cancel
					}).then({x:window.GameContainer.visibleWidth/2  - 150, alpha:1}, 1500, Easings.QUAD_IN_OUT).then({x:window.GameContainer.visibleWidth/2 - 150 + 20}, 10000, Easings.LINEAR).then({x:window.GameContainer.visibleWidth/2 + 500, alpha:0}, 800, Easings.QUAD_IN_OUT).play();
				}else{
					tokenCallout.tween = new Tween(tokenCallout, {x:window.GameContainer.visibleWidth/2  - 150, alpha:1}, 800, Easings.QUAD_IN_OUT).then({x:window.GameContainer.visibleWidth/2 - 150 + 20}, 1300, Easings.LINEAR).then({x:window.GameContainer.visibleWidth/2 + 500, alpha:0}, 500, Easings.QUAD_IN_OUT).then(function(){
						tokenCallout.x = window.GameContainer.visibleWidth/2 - 400;
					}).play();
				}
				
			} else {
				addScore(artistRingPts);
				tokenCallout.setFillText(collectedBalls+'/'+totalBalls);
				tokenCallout.tween = new Tween(tokenCallout, {x:window.GameContainer.visibleWidth/2  - 150, alpha:1}, 800, Easings.QUAD_IN_OUT).then({x:window.GameContainer.visibleWidth/2 - 150 + 20}, 1300, Easings.LINEAR).then({x:window.GameContainer.visibleWidth/2 + 500, alpha:0}, 500, Easings.QUAD_IN_OUT).then(function(){
					tokenCallout.x = window.GameContainer.visibleWidth/2 - 400;
				}).play();
			}
			
			this.collected = true;
			pickupCollectFunc.call(this);
			sfxSprite.play('Sfx_token_pick_up_'+collectedBalls);
			
		}
		function artistMissFunc(){
			if (!this.collected) {
				failedBalls = true;
				collectedBalls = 0;
				console.log("FAILED ring chain!")
			}
		}
		function noteCollectFunc(){
			curIntensityNotes++;
			if (curIntensityNotes == notesToIntensityUp) {
				curIntensityNotes = 0;
				intensityUp();
			}
			notesOnScreen.splice(notesOnScreen.indexOf(this),1);
			notesCollected++;
			this.collected = true;//is this necessary? even if so, maybe move into pickupcollectfunc?
			pickupCollectFunc.call(this);
			addScore(notePts*multLevel);
			
			sfxSprite.play('Sfx_note_pick_up');
		}
		function noteMissFunc(){
			if (!this.collected && !isTop) {
				intensityDown();
			}else{
				notesCollected++;
				this.collected = true;//is this necessary? even if so, maybe move into pickupcollectfunc?
			}
		}
		function noteBonusCollectFunc(){
			curIntensityNotes++;
			if (curIntensityNotes == notesToIntensityUp) {
				curIntensityNotes = 0;
				intensityUp();
			}
			notesOnScreen.splice(notesOnScreen.indexOf(this),1);
			this.collected = true;//is this necessary? even if so, maybe move into pickupcollectfunc?
			pickupCollectFunc.call(this);
			addScore(noteBonusPts*multLevel);
			
			sfxSprite.play('Sfx_note_pick_up');
		}
		function noteBonusMissFunc(){
			//intensityDown();
		}		
		function intensityUp(){
			if (intensityLevel < maxIntensity) {
				intensityLevel++;
				if(GameData.get("muted", true) != "true" && currentLevel<=3){
					allMusicTracks[intensityLevel].fade(0,1,500);
				}
				updateMult();
				intensityShineFrac = 0;//start
				intensityShakeFrac = 0.99;//cancel
				
				sfxSprite.play('Sfx_combo_level_up_'+intensityLevel);
			}
		}
		function intensityDown(){
			curIntensityNotes = 0;
			if (intensityLevel > 0) {
				if(GameData.get("muted", true) != "true" && currentLevel<=3){
					allMusicTracks[intensityLevel].fade(1,0,500);
				}
				intensityLevel--;
				updateMult();
				intensityShakeFrac = 0;//start
				intensityShineFrac = 0.99;//cancel
				
				sfxSprite.play('Sfx_combo_level_down');
			}
			if(isTop){
				fallThrough();
			}
		}
		function fallThrough(){
			bonusEnded();
			topFailed = true;
		}
		function bonusEnded(){
			if(topFailed)return;

			isTop = false;

			if(GameData.get("muted", true) != "true"){
				allMusicTracks[0].fade(0,1,100);
				allMusicTracks[1].fade(1,0,100);
			}

			bgDark.loadBitmap('media/images/gameplay/layers/BG_dark_'+currentLevel+'.jpg');
			bgGlow.loadBitmap('media/images/gameplay/layers/BG_glow_'+currentLevel+'.jpg');

			invincibilityCountdown = 200;
		}
		function updateMult(){
			multLevel = intensityMults[intensityLevel];
			switch(multLevel){
				case 1:
					comboHolder.scaleX = 0.7;
					comboHolder.scaleY = 0.85;
					comboNumberFont.fillStyle = numberGradient1.export();
					comboNumberFont.strokeStyle = '#93ddff';
				break;
				case 2:
					for(var i = 0; i < notesOnScreen.length;i++){
						notesOnScreen[i].loadBitmap("media/images/gameplay/objects/pickup.png");
					}
					comboHolder.scaleX = 0.85;
					comboHolder.scaleY = 1;
					comboNumberFont.fillStyle = numberGradient2.export();
					comboNumberFont.strokeStyle = '#8cff53';
				break;
				case 4:
					for(var i = 0; i < notesOnScreen.length;i++){
						notesOnScreen[i].loadBitmap("media/images/gameplay/objects/pickupG.png");
					}
					comboHolder.scaleX = 1;
					comboHolder.scaleY = 1.15;
					comboNumberFont.fillStyle = numberGradient3.export();
					comboNumberFont.strokeStyle = '#f8ee7a';
				break;
			}
			comboNumber.setStrokeText('x',{"%%":multLevel});
		}
		
		var beamLoader = new DisplayObjectFixed();
		beamLoader.loadBitmap('media/images/gameplay/objects/beam.png');
		function artistballInit(){
			
			this.beams = [];
			for(var i=0; i<18; i++){
				var curBeamInner = new DisplayObjectFixed();
					curBeamInner.loadBitmap(beamLoader.data);
					curBeamInner.y = -3;
				var curBeamOuter = new DisplayObjectFixed();
					curBeamOuter.addChild(curBeamInner);
					curBeamOuter.x = 64;
					curBeamOuter.y = 64;
				this.addChild(curBeamOuter);
				this.beams.push(curBeamOuter);
			}
			
			var artistballDisp = new DisplayObjectFixed();
			artistballDisp.loadBitmap('media/images/gameplay/objects/artistball_'+curArtistChoice+((currentLevel <= 3)?"":"_2015")+'.png');
			this.addChild(artistballDisp);
			
		}
		function artistballDraw(){
			
			for(var i=0; i<this.beams.length; i++){
				
				var curBeam = this.beams[i];
				
				if (isBeatChange) {
					curBeam.startRot = Math.random()*360;
					curBeam.offRot = (Math.random()-0.5)*90;
					curBeam.baseScaleX = 1-(Math.random()*0.5);
					curBeam.baseScaleY = 1-(Math.random()*0.5);
				}
				
				var curvedFrac = 1-withinBeat;
				
				curBeam.rotation = curBeam.startRot + curBeam.offRot * curvedFrac;
				curBeam.scaleX = curBeam.baseScaleX * curvedFrac;
				curBeam.scaleY = curBeam.baseScaleY * curvedFrac;
				curBeam.alpha = curBeam;
				
			}
			
		}

		function goldtrailUpdateFunc(){
			this.children[0].alpha *= 0.98;
			//this.children[0].scaleX *= 0.99;
			this.children[0].scaleY *= 0.97;
			this.children[0].y = this.children[0].dataHeight*0.5 * Math.abs(this.children[0].scaleY - 1);
			this.children[0].x = this.children[0].dataWidth*0.5 * Math.abs(this.children[0].scaleX - 1);
		}
		
		
		var groundBuffer = Buffer.create('groundBuffer');
			groundBuffer.width = 220;
			groundBuffer.height = 495;
		var groundCtx = groundBuffer.getContext('2d');
		
		var groundSrc = new DisplayObjectFixed();
			if(currentLevel <= 3){
				groundSrc.loadBitmap('media/images/gameplay/objects/ground_frames.jpg');
			}else{
				groundSrc.loadBitmap('media/images/gameplay/objects/ground_frames_2015.jpg');
			}

		var groundBonusBuffer = Buffer.create('groundBonusBuffer');
			groundBonusBuffer.width = 220;
			groundBonusBuffer.height = 153;
		var groundBonusCtx = groundBonusBuffer.getContext('2d');
		
		var groundBonusSrc = new DisplayObjectFixed();
			groundBonusSrc.loadBitmap('media/images/gameplay/objects/ground_bonus_frames.jpg');

	
		
		var platformBuffer = Buffer.create('platformBuffer');
			platformBuffer.width = 55;
			platformBuffer.height = 195;
		var platformCtx = platformBuffer.getContext('2d');
		
		var platformFrame = 0;
		
		var platformSrc = new DisplayObjectFixed();
			if(currentLevel <= 3){
				platformSrc.loadBitmap('media/images/gameplay/objects/platform_frames.png');
			}else{
				platformSrc.loadBitmap('media/images/gameplay/objects/platform_frames_2015.png');
			}
		
		var groundEntityType = new EntityFactory({
			buffer:groundBuffer,
			colWR:tileSize*4,
			colHD:tileSize,
			landFunc:platformLandFunc,
			collideFunc:obstacleCollideFunc,
			holder:world,
			autoRemove:true,
			forcePreciseJump:true
		});
		var groundBonusEntityType = new EntityFactory({
			buffer:groundBonusBuffer,
			colWR:tileSize*4,
			colHD:tileSize,
			landFunc:platformBonusLandFunc,
			collideFunc:obstacleBonusCollideFunc,
			holder:world,
			autoRemove:true,
			forcePreciseJump:true
		});
		var platformEntityType = new EntityFactory({
			buffer:platformBuffer,
			colWR:tileSize,
			colHD:tileSize,
			landFunc:platformLandFunc,
			holder:platformHolder,
			autoRemove:true,
			forcePreciseJump:true
		});
		var noteEntityType = new EntityFactory({
			path:'pickup',
			offsetX:21,
			offsetY:21,
			colWR:tileSize,
			colHD:tileSize,
			collideFunc:noteCollectFunc,
			missedFunc:noteMissFunc,
			holder:world,
			autoRemove:true
		});
		var noteBonusEntityType = new EntityFactory({
			path:'pickup_bonus',
			offsetX:21,
			offsetY:21,
			colWR:tileSize,
			colHD:tileSize,
			collideFunc:noteBonusCollectFunc,
			missedFunc:noteBonusMissFunc,
			holder:world,
			autoRemove:true
		});
		var lifeEntityType = new EntityFactory({
			path:'lifeicon',
			offsetX:143,
			offsetY:143,
			colWR:tileSize,
			colHD:tileSize,
			collideFunc:lifeCollectFunc,
			holder:world,
			autoRemove:true
		});
		var pickupFlashType = new EntityFactory({
			initFunc:pickupFlashInit,
			updateFunc:pickupFlashUpdate,
			holder:world,
		});
		var artistRingEntityType = new EntityFactory({
			path:'artistball_'+curArtistChoice+((currentLevel <= 3)?"":"_2015"),
			colWR:30,
			colWL:30,
			colHU:30,
			colHD:30,
			offsetX:53,
			offsetY:53,
			holder:world,
			autoRemove:true,
			collideFunc:artistCollectFunc,
			missedFunc:artistMissFunc
		});
		var artistBallEntityType = new EntityFactory({
			colWR:30,
			colWL:30,
			colHU:30,
			colHD:30,
			offsetX:32,
			offsetY:32,
			holder:world,
			autoRemove:true,
			extraDisposeW:241-64,
			initFunc:artistballInit,
			drawFunc:artistballDraw,
			collideFunc:artistCollectFunc,
			missedFunc:artistMissFunc
		});
		var mineEntityType = new EntityFactory({
			path:'mine',
			spriteInfo:{width:75,height:75,initAnim:{frames:[0,1,2,3],interframe:1}},
			offsetX:10,
			offsetY:10,
			colWR:tileSize-10,
			colWL:-10,
			colHD:tileSize,
			collideFunc:obstacleCollideFunc,
			holder:world,
			autoRemove:true
		});
		var invincibilityEntityType = new EntityFactory({
			path:'invincibility',
			colWR:tileSize,
			colHD:tileSize,
			offsetX:143,
			offsetY:143,
			collideFunc:invincibilityCollideFunc,
			holder:world,
			autoRemove:true
		});
		var bounceEntityType = new EntityFactory({
			path:'bounce',
			colWR:tileSize,
			colHD:tileSize,
			offsetX:117,
			offsetY:110,
			collideFunc:bounceCollideFunc,
			holder:world,
			autoRemove:true
		});
		var bonusEndEntityType = new EntityFactory({
			colWR:tileSize,
			colHD:tileSize*2,
			/*offsetX:30,
			offsetY:30,*/
			collideFunc:bonusEndCollideFunc,
			holder:world,
			autoRemove:true
		});
		var goldtrailerEntityType = new EntityFactory({
			path:'goldtrail',
			spriteInfo:{width:90,height:145,initAnim:{frames:[0,1,2],interframe:3, oneshot:true}},
			updateFunc:goldtrailUpdateFunc,
			holder:platformHolder,
			autoRemove:true
		});
		
		
		
		
		
		var thisRef = this;
		function playerHit(){
			
			thisRef.insideObstacle = true;
			
			if (thisRef.invincible) return;
			thisRef.invincible = true;
			playerPushY = -25;
			playerAnim.loadAnim(hurtAnim);
			isHurting = true;
			
			if (invincibilityCountdown == 0) {
				lives--;
				lives = Math.max(0,lives);
				updateLivesDisp();
				intensityDown();
				curIntensityNotes = 0;
				
				sfxSprite.play('Sfx_loosing_a_life');
			}
			
		}
		
		
		
		var lives = GameData.get("lives");
		var maxLives = GameData.get("lives");
		
		var lifeFull = new DisplayObjectFixed();
		lifeFull.loadBitmap('media/images/gameplay/ui/lives_full.png');
		
		var lifeEmpty = new DisplayObjectFixed();
		lifeEmpty.loadBitmap('media/images/gameplay/ui/lives_empty.png');
		
		var livesHolder = new DisplayObjectFixed();
		addCornerLocker(livesHolder,-1,-1,0,0);
		screenHolder.addChild(livesHolder);
		
		var lifeIcons = [];
		for (var i=0; i<lives; i++){
			var lifeIcon = new DisplayObjectFixed();
				lifeIcon.loadBitmap('media/images/gameplay/ui/lives_full.png');
				lifeIcon.x = 30+i*50;
				lifeIcon.y = 30;
				livesHolder.addChild(lifeIcon);
				lifeIcons.push(lifeIcon);
		}
		
		function updateLivesDisp(){
			for (var i=0; i<lifeIcons.length; i++) {
				lifeIcons[i].loadBitmap( ( (i>=lives) ? lifeEmpty : lifeFull ).data );
			}
		}
		
		
		
		var pauseBtn = new DisplayObjectFixed();
			pauseBtn.loadBitmap('media/images/buttons/btnPause.png');
		screenHolder.addChild(pauseBtn);
		addCornerLocker(pauseBtn,-1,-1,30,75);
		
		
		
		function getMouseXY(){
			if(window.GameContainer.browserInfo.touchDevice){
				if(window.GameContainer.mouse.touch.length > 0){
					return {
						x:window.GameContainer.mouse.touch[0].x,
						y:window.GameContainer.mouse.touch[0].y
					};
				} else {
					return false;
				}
			}else{
				return {
					x:window.GameContainer.mouse.x,
					y:window.GameContainer.mouse.y
				};
			}
		}
		
		var mouseWasDown = false;
		function isMouseOver(theDO){
			
			// make recursive?
			
			var mouseCoords = getMouseXY();
			
			if (mouseCoords) {
				
				var xpos = mouseCoords.x;
				var ypos = mouseCoords.y;
				
				if (xpos < theDO.x) return false;
				if (xpos > theDO.x + theDO.width) return false;
				if (ypos < theDO.y) return false;
				if (ypos > theDO.y + theDO.height) return false;
				
				return true;
				
			} else {
				
				return false;
				
			}
			
		}
		
		
		
		
		if (keyboardMode) {
			keyStates = new KeyStates();
		} else {
			
		}
		
		logicHeartbeats = 0;
		
		
		
		
		
		
		
		var songStartTime;
		var songStarted = false;
		var songStartRequested = false;
		var songOver = false;
		
		function songEndedFunc() {
			if(currentLevel == 0)return;
			Background.set(thisRef.engine.canvas.toDataURL("image/png"));
			endLevel(END_WIN);
			songOver = true;
		}
		var songsLoaded = 0;
		function oneSongLoadedFunc(){
			songsLoaded++;
		}
		
		
		
		
		// Howler has to be loaded AFTER title-screen audio tag, in order to avoid firefox bug
		var numBonusTracks = (currentLevel<=3)?2:1;
		var bgMusicBase;
		var allMusicTracks = [];
		
		var intensityLevel = 0;
		var maxIntensity;
		var dispIntensity = 0;
		
		var sfxSprite;
		
		var bpm;
		var playerWalkSpeed;
		var playerPushX;
		var track;
		
		var totalBalls = 0;
		var ballsCount = 0;
		var failedBalls = false;
		var collectedBalls = 0;
		var dispBalls = 0;
		
		var levelLayout;
		var mapPath = 'level'+currentLevel;
		require(["libs/text!game/maps/"+mapPath+".json"], function(mapSource){
			
			var mapJson = JSON.parse(mapSource);
			
			bpm = mapJson.bpm;
			track = mapJson.songfile;
			playerWalkSpeed = pixelsPerBeat / ((60/bpm)*logicHeartbeatsPerSecond);
			playerPushX = playerWalkSpeed;
			
			levelLayout = mapJson.layout;
			
			for(var barIndex=0; barIndex<levelLayout.length; barIndex++){
				var curBar = levelLayout[barIndex];
				if (curBar.beats) {
					for(var beatIndex=0; beatIndex<curBar.beats.length; beatIndex++){
						var curBeatList = curBar.beats[beatIndex];
						for(var beatItemIndex=0; beatItemIndex<curBeatList.length; beatItemIndex++){
							var curBeatItem = curBeatList[beatItemIndex];
							if (curBeatItem.ball !== undefined) totalBalls++;
						}
					}
				}
				if(curBar.bonus){
					if (curBar.bonus.beats) {
						for(var beatIndex=0; beatIndex<curBar.bonus.beats.length; beatIndex++){
							var curBeatList = curBar.bonus.beats[beatIndex];
							for(var beatItemIndex=0; beatItemIndex<curBeatList.length; beatItemIndex++){
								var curBeatItem = curBeatList[beatItemIndex];
								if (curBeatItem.ball !== undefined) totalBalls++;
							}
						}
					}
				}
			}


		
			for(var i=0; i<(numBonusTracks+1); i++){
				//var currentLevel = GameData.get("selectedLevel");
				//var track = GameData.get("level"+4,true);
				//console.log(track);
				var pathbase = 'media/music/'+track+'_';
				
				var curTrackOptions = {
					urls:[
						pathbase+i+'.mp3',
						pathbase+i+'.ogg'
					],
					onload:oneSongLoadedFunc
				};
				if (i==0) {
					curTrackOptions.onend = songEndedFunc;
				} else {
					curTrackOptions.volume = 0;
				}
				var curTrack = new Howl(curTrackOptions);
				if(GameData.get("muted", true) == "true"){
					curTrack.volume(0);
				}
				allMusicTracks.push(curTrack);
				
			}
			bgMusicBase = allMusicTracks[0];
			GameData.set("music",allMusicTracks);
			maxIntensity = /*allMusicTracks.length-1*/2;
			
			
			sfxSprite = new Howl({
				urls:['media/music/audiosprite.mp3','media/music/audiosprite.ogg'],
				onload:oneSongLoadedFunc,
				sprite:{
					Sfx_combo_level_down:[0,1470],
					Sfx_combo_level_up_1:[1570,1750],
					Sfx_combo_level_up_2:[3420,1740],
					Sfx_combo_level_up_3:[5260,1750],
					Sfx_life_pick_up:[7110,2120],
					Sfx_loosing_a_life:[9330,1630],
					Sfx_note_pick_up:[10976,209],
					Sfx_token_pick_up_1:[11260,1240],
					Sfx_token_pick_up_2:[12600,1250],
					Sfx_token_pick_up_3:[13950,1250],
					Sfx_token_pick_up_4:[15300,1920]
				},
			});
			if(GameData.get("muted", true) == "true"){
				sfxSprite.volume(0);
			}
				

			
		});
		
		
		
		
		
		
		
		
		var countdownUntilSong = 30;
		
		console.log("HELLO FROM GAMEPLAY CONSTRUCTOR");
		
		this.onload = function(){
			console.log("GAME ONLOAD...");
			// song load starts in update, gameplay starts when it's finished
		};


		this.onresume = function(){
			this.unpause();
		};
		
		
		
		
		
		
		
		
		function millisToBeat(millis){
			return millis/1000/(60/bpm);
		}
		
		
		function updateKeys() {
			if (gameover) {
				jump_new = false;
				jump_held = false;
			} else {
				if (keyboardMode) {
					jump_new = keyStates.keysDown[38];
					jump_held = keyStates.keysHeld[38];
					keyStates.update();
				} else {
					jump_new = window.GameContainer.mouse.down && !jump_held;
					jump_held = window.GameContainer.mouse.down;
				}
			}
		}
		
		
		
		var gamePaused = true;
		var pausedSeekTime = 0;
		this.emergencyPause = function(){
			if(gamePaused)return;
			this.pause();
		}
		this.manualPause = function(){
			if(gamePaused)return;
			console.log("manual pause");
			this.pause();
		}
		this.pause = function(){
			if(gamePaused)return;
			for(var i=0; i<allMusicTracks.length; i++) allMusicTracks[i].pause();
			pausedSeekTime = logicHeartbeats/logicHeartbeatsPerSecond;
			gamePaused = true;
			GameData.set("sprite",sfxSprite);
			setTimeout(function(){
				Background.set(thisRef.engine.canvas.toDataURL("image/png"));
				EventManager.broadcast("showPopup", "popupPause");
			},0);
		}
		this.unpause = function(){
			for(var i=0; i<allMusicTracks.length; i++) {
				allMusicTracks[i].play();
				allMusicTracks[i].pos(pausedSeekTime);//only in emergencyPause, maybe?
			}
			gamePaused = false;
			songStartTime = Date.now() - pausedSeekTime;
		}
		this.debugSkip = function(){
			var debugBeat = GameData.get("debugBeat");
			var seekTime = 60/bpm*debugBeat;
			for(var i=0; i<allMusicTracks.length; i++) {
				allMusicTracks[i].play();
				allMusicTracks[i].pos(seekTime);//only in emergencyPause, maybe?
			}
			gamePaused = false;
			songStartTime = Date.now() - seekTime;
		}
		
		var curScreenW = 0;
		var curScreenH = 0;
		
		this.update = function(){
			
			
			
			
			if ( (window.GameContainer.visibleWidth != curScreenW) || (window.GameContainer.visibleHeight != curScreenH) ) {
				curScreenW = window.GameContainer.visibleWidth;
				curScreenH = window.GameContainer.visibleHeight;
				this.onResize();
			}
			
			
			var mouseIsDown = window.GameContainer.mouse.down;
			var isClick = mouseIsDown && !mouseWasDown;
			mouseWasDown = mouseIsDown;
			
			
			
			/*
			if (isClick) {
				var tempClickCoords = getMouseXY();
				if (tempClickCoords.x < 379) intensityDown();
				if (tempClickCoords.x > 575) intensityUp();
			}
			*/
			
			//Tutorial
			if(currentLevel == 0){
				if(curBeat > tutDist[currentTut] && currentTut < tutDist.length){
					currentTut++;
				}
				if(currentTut == tutDist.length){
					if(GameData.get("tutDone", true) == "false"){
						GameData.set("tutDone", "true", true);
						GameData.set("selectedLevel", 4);
						EventManager.broadcast("gotoScreen", "gameplay");
					}else{
						EventManager.broadcast("gotoScreen", "chooseLevel");
					}
				}
				if(tutLayer.children[currentTut] && tutLayer.children[currentTut].alpha == 0){
					if(currentTut == 0){
						tutLayer.tween = new Tween(tutLayer.children[currentTut], {alpha:1}, 500, Easings.QUAD_IN_OUT).play();
					}else{
						tutLayer.tween = new Tween(tutLayer.children[currentTut-1], {alpha:0}, 500, Easings.QUAD_IN_OUT).then(function(){
							tutLayer.tween = new Tween(tutLayer.children[currentTut], {alpha:1}, 500, Easings.QUAD_IN_OUT).play();
						}).play();
					}
				}
			}
			//console.log(curBeat);
			
			if (gameover) return;
			
			
			if (!levelLayout) return;
			
			
			if (!songStartRequested) {
				if (songsLoaded < allMusicTracks.length+1) {//plus one because it's also waiting for sound sprite
					console.log("waiting for songs to load...");
					return;
				} else {
					console.log("REQUESTING SONG START!");
					songStartRequested = true;
					if(GameData.get("debugBeat") != null){
						this.debugSkip();
					}else{	
						this.unpause();
					}
				}
			}
			
			
			
			
			if (gamePaused) {
				
				// always unpause as soon as able to; if the update function has been called, that means the pause menu is closed
				//this.unpause();
				
			} else {
				
				
				if (isClick) {
					if (isMouseOver(pauseBtn)) {
						this.manualPause();
					}
				}
				
				
				
				// lock this behind something in the config!
				debugSlowmo = false;
				if (keyboardMode) {
					if (keyStates.keysHeld[16]) {
						debugSlowmo = true;
					}
				}
				
				if (debugSlowmo) {
					if (debugSlowmoCounter > 0) {
						debugSlowmoCounter--;
						return;
					} else{
						debugSlowmoCounter = 60;
					}
				} else {
					debugSlowmoCounter = 0;
				}
				//
				
				
				
				
				
				now = Date.now();
				
				
				if ( bgMusicBase.pos() != lastSongTime ) {
					if (songOver) return;
					lastSongTime = bgMusicBase.pos();
					songStartTime = now - lastSongTime*1000;
					songStarted = true;
				}
				
				
				
				if (!songStarted) return;
				
				
				
				var songTime = now-songStartTime;
				curBeat = millisToBeat(songTime);
				withinBeat = curBeat%1;
				beatNum = Math.floor(curBeat);
				
				
				
				var worldTime = songTime / 1000;
				var goalHeartbeats = Math.floor(worldTime * logicHeartbeatsPerSecond);
				
				if ( (goalHeartbeats - logicHeartbeats) > maxHeartbeatsPerFrame && GameData.get("debugBeat") == null) {
					
					console.log("HUGE LAG HAPPENED! PAUSING");
					this.emergencyPause();
					
				} else {
					
					if (logicHeartbeats < goalHeartbeats) {
						while (logicHeartbeats < goalHeartbeats) {
							this.updateLogic();
							logicHeartbeats++;
							//
							if (gameover) return;
							//
						}
						this.updateDraw();
						lastBeat = curBeat;
					}
					
				}
				
				
			}
			
			
		}
		
		this.onResize = function(){
			for(var i=0; i<cornerLockers.length; i++){
				var curCornerLocker = cornerLockers[i];
					curCornerLocker.cornerDO.x = 1136/2 + curCornerLocker.xSide * curScreenW/2 + curCornerLocker.xOff;
					curCornerLocker.cornerDO.y =  672/2 + curCornerLocker.ySide * curScreenH/2 + curCornerLocker.yOff;
			}
		}
		
		
		
		var debugSlowmo;
		function debugLog(msg){
			if (debugSlowmo) console.log(msg);
		}
		
		
		
		
		this.updateLogic = function(){
			
			
			
			updateKeys();
			
			
			wasOnGround = onGround;
			var playerWasGoingUp = (playerPushY < 0);
			
			if (onGround && /*jump_new*/ jump_held) {
				playerPushY = -jumpSpeed;
				if (!isHurting) playerAnim.loadAnim(jumpAnim);
			}
			
			
			playerPushY += gravity;
			playerY += playerPushY;
			
			playerX += playerWalkSpeed;
			
			if (!isHurting && playerWasGoingUp && (playerPushY >= 0)) playerAnim.loadAnim(fallAnim);
			
			var spawnCodes = [
				"plat",
				"spike",
				"note",
				"noteBonus",
				"ball",
				"inv",
				"life",
				"bounce",
				"bonusEnd"
			];
			var spawnTypes = [
				platformEntityType,
				mineEntityType,
				noteEntityType,
				noteBonusEntityType,
				artistBallEntityType,
				invincibilityEntityType,
				lifeEntityType,
				bounceEntityType,
				bonusEndEntityType
			];
			
			
			
			while(cameraX + 1136 + 200 > nextTileIndex*4*tileSize){
				
				if (nextTileIndex >= levelLayout.length) break;
				
				var curTileInfo = levelLayout[nextTileIndex];
				
				if (curTileInfo.yOff) curTileY -= curTileInfo.yOff;
				
				groundEntityType.spawn(nextTileIndex*4*tileSize,curTileY*tileSize);
				
				if (curTileInfo.beats) {
					for(var beatIndex=0; beatIndex<curTileInfo.beats.length; beatIndex++){
						
						var curBeatList = curTileInfo.beats[beatIndex];
						
						for (var beatItemIndex=0; beatItemIndex<curBeatList.length; beatItemIndex++) {
							
							var curBeatItem = curBeatList[beatItemIndex];
							
							for(var typeIndex=0; typeIndex<spawnCodes.length; typeIndex++){
								var curCode = spawnCodes[typeIndex];
								if (curBeatItem[curCode] !== undefined) {
									
									var curType = spawnTypes[typeIndex];
									
									if (curType == artistBallEntityType) {
										if (failedBalls) {
											curType = null;
										} else {
											ballsCount++;
											if (ballsCount<totalBalls) curType = artistRingEntityType;
										}
									}
									
									if (curType != null)curType.spawn((nextTileIndex*4+beatIndex)*tileSize, (curTileY-curBeatItem[curCode]-1)*tileSize);
									
								}
							}
							
						}
						
					}
				}

				if(curTileInfo.bonus){
					if (curTileInfo.bonus.yOff) curTileBonusY -= curTileInfo.bonus.yOff;
				
					groundBonusEntityType.spawn(nextTileIndex*4*tileSize,curTileBonusY*tileSize);

					if (curTileInfo.bonus.beats) {
						for(var beatIndex=0; beatIndex<curTileInfo.bonus.beats.length; beatIndex++){
							
							var curBeatList = curTileInfo.bonus.beats[beatIndex];
							
							for (var beatItemIndex=0; beatItemIndex<curBeatList.length; beatItemIndex++) {
								
								var curBeatItem = curBeatList[beatItemIndex];
								
								for(var typeIndex=0; typeIndex<spawnCodes.length; typeIndex++){
									var curCode = spawnCodes[typeIndex];
									if (curBeatItem[curCode] !== undefined) {
										
										var curType = spawnTypes[typeIndex];
										
										if (curType == artistBallEntityType) {
											if (failedBalls) {
												curType = null;
											} else {
												ballsCount++;
												if (ballsCount<totalBalls) curType = artistRingEntityType;
											}
										}
										
										if (curType != null)curType.spawn((nextTileIndex*4+beatIndex)*tileSize, (curTileBonusY-curBeatItem[curCode]-1)*tileSize);
										
									}
								}
							}
						}
					}
				}
				
				nextTileIndex++;
				
			}
			
			
			
			onGround = false;
			
			
			this.insideObstacle = false;
			
			
			for(var obstacleIndex=activeEntities.length-1; obstacleIndex>=0; obstacleIndex--){
				var curEntity = activeEntities[obstacleIndex];
				if (!curEntity.autoRemove) continue;
				
				if (curEntity.x+curEntity.width+curEntity.extraDisposeW < cameraX){
					if(curEntity.debugPath == "pickup"){
						notesOnScreen.shift();
					}
					curEntity.kill();
				} 
			}
			
			
			for(var obstacleIndex=activeEntities.length-1; obstacleIndex>=0; obstacleIndex--) {
				var curEntity = activeEntities[obstacleIndex];
				if (!curEntity.updateFunc) continue;
				
				curEntity.updateFunc();
			}
			
			for(var obstacleIndex=activeEntities.length-1; obstacleIndex>=0; obstacleIndex--) {
				
				var curEntity = activeEntities[obstacleIndex];
				if (!curEntity.landFunc) continue;

				if (playerX <= curEntity.x+curEntity.offsetX-curEntity.colWL) continue;
				if (playerX >= curEntity.x+curEntity.offsetX+curEntity.colWR) continue;
				if ( (playerY >= curEntity.y+curEntity.offsetY-curEntity.colHU) && (playerY-playerPushY <= curEntity.y+curEntity.offsetY-curEntity.colHU)) {
					curEntity.landFunc();
				}
				
			}
			
			for(var obstacleIndex=activeEntities.length-1; obstacleIndex>=0; obstacleIndex--) {
				
				var curEntity = activeEntities[obstacleIndex];
				if (!curEntity.collideFunc) continue;

				if (playerX <= curEntity.x+curEntity.offsetX-curEntity.colWL) continue;
				if (playerX >= curEntity.x+curEntity.offsetX+curEntity.colWR) continue;
				
				if (playerY <= curEntity.y+curEntity.offsetY-curEntity.colHU) continue;
				if (playerY-playerColH >= curEntity.y+curEntity.offsetY+curEntity.colHD) continue;
				
				curEntity.collideFunc();
			}
			
			for(var obstacleIndex=activeEntities.length-1; obstacleIndex>=0; obstacleIndex--) {
				
				var curEntity = activeEntities[obstacleIndex];
				if (!curEntity.missedFunc) continue;
				
				if (curEntity.alreadyMissed) continue;
				
				if (playerX < curEntity.x+curEntity.offsetX+curEntity.colWR) continue;
				
				curEntity.alreadyMissed = true;
				curEntity.missedFunc();
				
			}
			
			
			for(var animatorIndex=animators.length-1; animatorIndex>=0; animatorIndex--){
				animators[animatorIndex].update();
			}
			
			
			if (!this.insideObstacle) this.invincible = false;
			
			
			
			
			
			if (invincibilityCountdown > 0) {
				invincibilityCountdown--;
				playerAnim.holderDO.alpha = ((invincibilityCountdown%6) == 1) ? ((invincibilityCountdown > 25) ? 0 : 0.5) : 1 ;
			}
			
			
			if (onGround) {
				if (playerPushY > 0) playerPushY = 0;
				if (!jump_held) playerCanJump = true;
			}
			
			
			
			var comboScale = 1;
			if (intensityShineFrac < 1) {
				intensityShineFrac += 0.03;
				if (intensityShineFrac > 1) intensityShineFrac = 1;
				comboShine.alpha = 1-Math.pow(intensityShineFrac,6);
				comboShine.x = 445 + Math.pow(intensityShineFrac,0.2)*220 + comboShineOffX;
				comboShine.rotation = 1080*Math.pow(intensityShineFrac,0.1);
				comboScale = (1+0.2*intensityLevel) + 0.5*(1-Math.pow(intensityShineFrac,0.5));//use this eventually?
			}else{
				addCornerLocker(comboShine,0,-1,-60,55);
				comboShineOffX = 0;
			}
			if (intensityShakeFrac < 1) {
				intensityShakeFrac += 0.05;
				if (intensityShakeFrac > 1) intensityShakeFrac = 1;
				comboHolder.x = (1136/2 + 65) + Math.sin(intensityShakeFrac*20)*(1-Math.pow(intensityShakeFrac,2))*30;
				comboScale = 1;
			}
			
			
			
			var cameraGoalX = playerX - cameraLeadDistX;
			var cameraGoalY = playerY - cameraLeadDistY;
			
			if (cameraGoalX < 0) cameraGoalX = 0;
			
			cameraX += (cameraGoalX - cameraX) * 0.5;
			cameraY += (cameraGoalY - cameraY) * 0.2;
			
			playerAnim.holderDO.x = playerX;
			playerAnim.holderDO.y = playerY;

			if(multLevel == 4){
				if(curStep >= stepGoal){
					goldtrailerEntityType.spawn(playerAnim.holderDO.x-50,playerAnim.holderDO.y-140);
					curStep = 0;
				}else{
					curStep++
				}
			}
			
			world.x = -cameraX;
			world.y = -cameraY;
			
			
			
			if (lives == 0 && GameData.get("debugBeat") == null){
				Background.set(thisRef.engine.canvas.toDataURL("image/png"));
				endLevel(END_FAIL);
			} 
			
			
			
			dispIntensity += (intensityLevel - dispIntensity) * 0.2;
			dispBalls += (collectedBalls - dispBalls) * 0.05;
			
			
		};
		
		this.updateDraw = function(){
			
			oldTwoBeat = (lastBeat+rayOffset)/2;
			twoBeat = (curBeat+rayOffset)/2;
			twoBeatChange = (Math.floor(oldTwoBeat) != Math.floor(twoBeat));
			
			withinTwobeat = twoBeat%1;
			curvedTwobeat = Math.pow(withinTwobeat,0.3);//1-Math.pow(Math.cos(withinTwobeat*Math.PI)*0.5+0.5,4);
			
			var softTwoFade = (1-(Math.cos(Math.pow(withinTwobeat,0.5)*2*Math.PI)*0.5+0.5)) * 0.5;
			
			if (twoBeatChange) raysActive = (intensityLevel>1);
			for(var i=0; i<sunrays.length; i++) {
				if (twoBeatChange) sunrays[i].randomizeBeam();
				sunrays[i].update();
			}
			sunrayLayer.alpha = raysActive ? softTwoFade : 0 ;
			
			var curLerpLow = Math.floor(dispIntensity);
			var curLerpFrac = dispIntensity % 1;
			if (curLerpLow == 2) {
				// there's nowhere to lerp up to at the max,
				// so just stay at the top range of the level below
				curLerpLow = 1;
				curLerpFrac = 1;
			}
			
			var glowMin = glowAmts[curLerpLow][0] + (glowAmts[curLerpLow+1][0] - glowAmts[curLerpLow][0]) * curLerpFrac;
			var glowMax = glowAmts[curLerpLow][1] + (glowAmts[curLerpLow+1][1] - glowAmts[curLerpLow][1]) * curLerpFrac;
			
			bgGlow.alpha = glowMin + softTwoFade * glowMax;
			
			//Math.pow(dispBalls / totalBalls, 4);
			if(dispBalls > 3){
				bgArtist.alpha = 1;
			}else if(dispBalls > 2){
				bgArtist.alpha = 0.6;
			}else if(dispBalls > 1){
				bgArtist.alpha = 0.4;
			}else if(dispBalls > 0){
				bgArtist.alpha = 0.2;
			}

			//console.log(bonusGlow.alpha);
			if(bonusGlow.alpha>0){
				bonusGlow.alpha *= 0.95; 
				if(bonusGlow.alpha <= 0.05)bonusGlow.alpha = 0;
			}
			
			
			isBeatChange = (Math.floor(lastBeat) != Math.floor(curBeat));
			
			if (isBeatChange) {
				platformFrame = 0;
				if ((Math.floor(curBeat)%2) == 1) {
					scrambleStars();
					starsActive = (intensityLevel > 0);
				}
			}
			
			
			var offbeat = ((curBeat-1)/2)%1;
			bgStarsHolder.alpha = starsActive ? Math.pow(1-offbeat,4) : 0;
			
			
			// wipe if transparent...
			groundCtx.drawImage(groundSrc.data,-220*Math.floor(withinBeat*6),0);
			groundBonusCtx.drawImage(groundBonusSrc.data,-220*Math.floor(withinBeat*6),0);
			
			platformCtx.clearRect(0,0,55,195);
			platformCtx.drawImage(platformSrc.data,-55*Math.floor(withinBeat*5),0);
			
			
			for(var obstacleIndex=activeEntities.length-1; obstacleIndex>=0; obstacleIndex--) {
				var curEntity = activeEntities[obstacleIndex];
				if (!curEntity.drawFunc) continue;
				
				curEntity.drawFunc();
			}
			
			
		};
		
		this.onunload = function(){
			if (keyboardMode) keyStates.dispose();
			
			Buffer.kill('groundBuffer');
			Buffer.kill('groundBonusBuffer');
			Buffer.kill('platformBuffer');

			Buffer.kill("fontGradient");
			Buffer.kill("TextField_comboLabel");
			Buffer.kill("TextField_comboNumber");
			Buffer.kill("TextField_tokenCallout");
			Buffer.kill("TextField_scoreDisp");
			Buffer.kill("media/images/artists/levelToken1Inactive.png");
			Buffer.kill("media/images/artists/levelToken2Inactive.png");
			Buffer.kill("media/images/artists/levelToken3Inactive.png");
			Buffer.kill("media/images/artists/levelToken4Inactive.png");
			Buffer.kill("media/images/artists/levelToken5Inactive.png");
			Buffer.kill("media/images/gameplay/layers/BG_dark_bonus.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_glow_bonus.jpg");
			Buffer.kill("media/images/gameplay/layers/star.png");
			Buffer.kill("media/images/gameplay/layers/bonus_overlay.png");
			Buffer.kill("media/images/gameplay/ui/combosparkle.png");
			Buffer.kill("media/images/gameplay/objects/Ardy_Anim.png");
			Buffer.kill("media/images/gameplay/layers/sunray.png");
			Buffer.kill("media/images/gameplay/objects/pickup_flash.png");
			Buffer.kill("media/images/gameplay/objects/beam.png");
			Buffer.kill("media/images/gameplay/objects/ground_frames_2015.jpg");
			Buffer.kill("media/images/gameplay/objects/ground_bonus_frames.jpg");
			Buffer.kill("media/images/gameplay/objects/platform_frames_2015.png");
			Buffer.kill("media/images/gameplay/objects/pickup_bonus.png");
			Buffer.kill("media/images/gameplay/objects/lifeicon.png");
			Buffer.kill("media/images/gameplay/objects/mine.png");
			Buffer.kill("media/images/gameplay/objects/invincibility.png");
			Buffer.kill("media/images/gameplay/objects/bounce.png");
			Buffer.kill("media/images/gameplay/objects/goldtrail.png");
			Buffer.kill("media/images/gameplay/ui/lives_full.png");
			Buffer.kill("media/images/gameplay/ui/lives_empty.png");
			Buffer.kill("media/images/gameplay/objects/pickupG.png");
			Buffer.kill("media/images/gameplay/objects/pickup.png");

			Buffer.kill("media/images/gameplay/layers/BG_dark_1.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_dark_2.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_dark_3.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_dark_4.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_dark_5.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_dark_6.jpg");

			Buffer.kill("media/images/gameplay/layers/BG_glow_1.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_glow_2.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_glow_3.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_glow_4.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_glow_5.jpg");
			Buffer.kill("media/images/gameplay/layers/BG_glow_6.jpg");

			Buffer.kill("media/images/gameplay/layers/artist_1_2015.png");
			Buffer.kill("media/images/gameplay/layers/artist_2_2015.png");
			Buffer.kill("media/images/gameplay/layers/artist_3_2015.png");
			Buffer.kill("media/images/gameplay/layers/artist_4_2015.png");
			Buffer.kill("media/images/gameplay/layers/artist_5_2015.png");
			Buffer.kill("media/images/gameplay/layers/artist_6_2015.png");

			Buffer.kill("media/images/gameplay/objects/artistball_1_2015.png");
			Buffer.kill("media/images/gameplay/objects/artistball_2_2015.png");
			Buffer.kill("media/images/gameplay/objects/artistball_3_2015.png");
			Buffer.kill("media/images/gameplay/objects/artistball_4_2015.png");
			Buffer.kill("media/images/gameplay/objects/artistball_5_2015.png");
			Buffer.kill("media/images/gameplay/objects/artistball_6_2015.png");

			Buffer.kill("media/images/screens/win/star2015_1.png");
			Buffer.kill("media/images/screens/win/star2015_2.png");
			Buffer.kill("media/images/screens/win/star2015_3.png");

			Buffer.kill("media/images/screens/chooseLevel/panelLevel1_2015.png");
			Buffer.kill("media/images/screens/chooseLevel/panelLevel2_2015.png");
			Buffer.kill("media/images/screens/chooseLevel/panelLevel3_2015.png");

			Buffer.kill("media/images/screens/chooseLevel/overlayLevel1_2015.png");
			Buffer.kill("media/images/screens/chooseLevel/overlayLevel2_2015.png");
			Buffer.kill("media/images/screens/chooseLevel/overlayLevel3_2015.png");


			for(var i=0; i<allMusicTracks.length; i++){
				allMusicTracks[i].unload();
				delete allMusicTracks[i];
			} 
			allMusicTracks = [];
			sfxSprite.unload();
			delete sfxSprite;
			delete allMusicTracks;
		};
		
		this.binds = {};
		
	};
	return screen;
});