define("screens/popupFail", [
	"utils/Background",
	"sdk/EventManager",
	"sdk/DisplayObject",
	"sdk/TextField",
	"sdk/GameData",
	"sdk/Sound",
	"sdk/Tween",
	"text/Font"
], 
function (Background, EventManager, DisplayObject, TextField, GameData, Sound, Tween, Font){
	var screen = function(){
		var thisRef = this,
			// Local
			// Styles
			overlayTitleFont = new Font("overlayTitleFont"),
			overlayTextFont = new Font("overlayTextFont"),
			// Elements
			overlayHolder = new DisplayObject("overlayHolder"),
			failHolder = new DisplayObject("failHolder"),
				overlayTitle = new TextField("overlayTitle"),
				overlayText = new TextField("overlayText"),
				ardy = new DisplayObject("ardy"),
				replayGlow= new DisplayObject("replayGlow"),
				replayBtn = new DisplayObject("replay"),
				quitGlow = new DisplayObject("quitGlow"),
				quitBtn = new DisplayObject("quit")
		;

		// Fonts

		overlayTitleFont.font = "56px NexaBold";
		overlayTitleFont.fillStyle = "#60D5FF";
		// Firefox display textBaseline = top differently that in other browser.
		// To fix it, we can simply set textBaseline = alphabetical and set a paddingTop equal to the font size. 
		overlayTitleFont.textBaseline = "alphabetical";
		overlayTitleFont.paddingTop = 56;
		overlayTitleFont.textAlign = "center";

		overlayTextFont.font = "25px NexaBold";
		overlayTextFont.fillStyle = "#60D5FF";
		// Firefox display textBaseline = top differently that in other browser.
		// To fix it, we can simply set textBaseline = alphabetical and set a paddingTop equal to the font size. 
		overlayTextFont.textBaseline = "alphabetical";
		overlayTextFont.paddingTop = 25;
		overlayTextFont.textAlign = "center";
		overlayTextFont.verticalSpacing = 5;
		overlayTextFont.textWrap = true;
		
		// Default

		// Overlay

		overlayHolder.loadBitmap("media/images/backgrounds/overlayBg.png");
		overlayHolder.alpha = 0;
		overlayHolder.setIndex(2);
		this.addChild(overlayHolder);

		failHolder.loadBitmap("media/images/screens/confirm/panel.png");
		failHolder.x = 292;
		failHolder.y = -643; // 20
		overlayHolder.addChild(failHolder);

		overlayTitle.width = 420;
		overlayTitle.height = 100;
		overlayTitle.x = (579 - overlayTitle.width) / 2;
		overlayTitle.y = 66;
		overlayTitle.font = overlayTitleFont;
		overlayTitle.setStrokeText("FailTitle");
		failHolder.addChild(overlayTitle);

		overlayText.width = 420;
		overlayText.height = 100;
		overlayText.x = (579 - overlayText.width) / 2;
		overlayText.y = 183;
		overlayText.font = overlayTextFont;
		overlayText.setStrokeText("FailText");
		failHolder.addChild(overlayText);

		ardy.loadBitmap("media/images/screens/fail/ardy.png");
		ardy.x = 491 - 292;
		ardy.y = 316 - 20;
		failHolder.addChild(ardy);

		replayGlow.loadBitmap("media/images/buttons/btnGlow.png");
		replayGlow.x = 849;
		replayGlow.y = 250;
		replayGlow.alpha = 0;
		overlayHolder.addChild(replayGlow);

		replayBtn.loadBitmap("media/images/buttons/btnReplay.png");
		replayBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnReplay.png");
		};
		replayBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnReplayPressed.png");
		};
		replayGlow.addChild(replayBtn);

		quitGlow.loadBitmap("media/images/buttons/btnGlow.png");
		quitGlow.x = 118;
		quitGlow.y = 250;
		quitGlow.alpha = 0;
		overlayHolder.addChild(quitGlow);

		quitBtn.loadBitmap("media/images/buttons/btnQuit.png");
		quitBtn.x = 40;
		quitBtn.y = 40;
		quitBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnQuit.png");
		};
		quitBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnQuitPressed.png");
		};
		quitGlow.addChild(quitBtn);

		// Methods
		
		this.onload = function(){
			thisRef.showOverlay();
			thisRef.bgMusic = new Howl({
				urls:['media/music/fail.mp3','media/music/fail.ogg'],
				autoplay:true
			});
			if(GameData.get("muted", true) == "true"){
				thisRef.bgMusic.volume(0);
			}
		};	
		
		this.update = function(){};
		
		this.onunload = function(){};

		this.showOverlay = function(){
			setTimeout(function(){ Tween.add(overlayHolder, { alpha: 1 }, 12, 0.5, function(){
				setTimeout(function(){ Tween.add(failHolder, { y: 20 }, 12, 0.5, function(){
					thisRef.btnFadeIn();
					Tween.add(quitGlow, { alpha: 1 }, 12, 0.5, function(){});
				}); }, 100);

			}); }, 50);
		};

		this.btnFadeIn = function(){
			setTimeout(function(){ Tween.add(replayGlow, { alpha: 1 }, 10, 0.75, function(){ thisRef.btnFadeOut(); }); }, 100);
		};

		this.btnFadeOut = function(){
			setTimeout(function(){ Tween.add(replayGlow, { alpha: 0.25 }, 10, 0.75, function(){ thisRef.btnFadeIn(); }); }, 100);
		};

		this.quit = function(){
			if(GameData.get("selectedLevel")>3){
				EventManager.broadcast("gotoScreen", "chooseLevel2015");
			}else{
				EventManager.broadcast("gotoScreen", "chooseLevel");
			}
			thisRef.bgMusic.unload();
		};

		this.replay = function(){
			EventManager.broadcast("gotoScreen", "gameplay");
			thisRef.bgMusic.unload();
		};
		
		this.binds = {
			"quit": thisRef.quit,
			"replay": thisRef.replay
		};
	};
	
	return screen;
});