define("screens/popupPause", [
	"utils/Background",
	"sdk/EventManager",
	"sdk/DisplayObject",
	"sdk/TextField",
	"sdk/GameData",
	"sdk/Tween",
	"text/Font"
], 
function (Background, EventManager, DisplayObject, TextField, GameData, Tween, Font)
{
	var screen = function()
	{
		var thisRef = this,
			// Local
			isOverlay = false,
			isHtp = false,
			htpTotalPage = 5,
			htpCurrentPage = 1,
			buttonEnabled = true,
			device = (GameContainer.browserInfo.touchDevice)? "mobile": "desktop",
			// Fonts
			overlayTitleFont = new Font("overlayTitleFont"),
			overlayTextFont = new Font("overlayTextFont"),
			overlayBtnFont = new Font("overlayBtnFont"),
			// Elements
			overlayHolder = new DisplayObject("overlayHolder"),
				pauseHolder = new DisplayObject("pauseHolder"),
					resumeGlow = new DisplayObject("resumeGlow"),
						resumeGlowImg = new DisplayObject("resumeGlowImg"),
						resumeBtn = new DisplayObject("resume"),
					muteBtn = new DisplayObject("mute"),
        		    unmuteBtn = new DisplayObject("unmute"),
        		    htpBtn = new DisplayObject("htp"),
					replayBtn = new DisplayObject("replay"),
					quitBtn = new DisplayObject("quit"),
				htpHolder = new DisplayObject("htpHolder"),
					htpTitle = new TextField("htpTitle"),
					htpClose = new DisplayObject("htpClose"),
					htpImage = new DisplayObject("htpImage"),
					htpPrev = new DisplayObject("prev"),
					htpNext = new DisplayObject("next"),
				quitHolder = new DisplayObject("quitHolder"),
					quitTitle = new TextField("quitTitle"),
					quitText = new TextField("quitText"),
					confirmQuitBtn = new DisplayObject("confirmQuit"),
					confirmQuitLabel = new TextField("confirmQuitLabel"),
					cancelQuitBtn = new DisplayObject("cancelQuit"),
					cancelQuitLabel = new TextField("cancelQuitLabel")
		;

		// Fonts

		overlayTitleFont.font = "40px NexaBold";
		overlayTitleFont.fillStyle = "#60D5FF";
		overlayTitleFont.textBaseline = "alphabetical";
		overlayTitleFont.paddingTop = 40;
		overlayTitleFont.textAlign = "center";

		overlayTextFont.font = "25px NexaBold";
		overlayTextFont.fillStyle = "#60D5FF";
		overlayTextFont.textBaseline = "alphabetical";
		overlayTextFont.paddingTop = 25;
		overlayTextFont.textAlign = "center";
		overlayTextFont.verticalSpacing = 5;
		overlayTextFont.textWrap = true;

		overlayBtnFont.font = "25px NexaBold";
		overlayBtnFont.fillStyle = "#FFF";
		overlayBtnFont.textBaseline = "middle";
		overlayBtnFont.paddingTop = 38; 
		overlayBtnFont.textAlign = "center";
		

		// Default

		overlayHolder.loadBitmap("media/images/backgrounds/overlayBg.png");
		this.addChild(overlayHolder);

		overlayHolder.addChild(pauseHolder);

		resumeBtn.loadBitmap("media/images/buttons/btnPauseResume.png");
		resumeBtn.x = 323;
		resumeBtn.y = -570;
		resumeBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnPauseResume.png");
		};
		resumeBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnPauseResumePressed.png");
		};
		pauseHolder.addChild(resumeBtn);

		muteBtn.loadBitmap("media/images/buttons/btnPauseSoundOn.png");
        muteBtn.x = 395;
        muteBtn.y = 672;
        muteBtn.alpha = 0;
        muteBtn.releasedState = function(){
            this.loadBitmap("media/images/buttons/btnPauseSoundOn.png");
        };
        muteBtn.pressedState = function(){
            this.loadBitmap("media/images/buttons/btnPauseSoundOnPressed.png");
        };
        pauseHolder.addChild(muteBtn);

        unmuteBtn.loadBitmap("media/images/buttons/btnPauseSoundOff.png");
        unmuteBtn.x = 395;
        unmuteBtn.y = 672;
        unmuteBtn.alpha = 0;
        unmuteBtn.releasedState = function(){
            this.loadBitmap("media/images/buttons/btnPauseSoundOff.png");
        };
        unmuteBtn.pressedState = function(){
            this.loadBitmap("media/images/buttons/btnPauseSoundOffPressed.png");
        };
        pauseHolder.addChild(unmuteBtn);

        if(GameData.get("muted", true) == "true"){
            unmuteBtn.alpha = 1;
        }
        else{
            muteBtn.alpha = 1;
        }

        htpBtn.loadBitmap("media/images/buttons/btnPauseHtp.png");
		htpBtn.x = 480;
		htpBtn.y = 672;
		htpBtn.releasedState = function()
		{
			this.loadBitmap("media/images/buttons/btnPauseHtp.png");
		};
		htpBtn.pressedState = function()
		{
			this.loadBitmap("media/images/buttons/btnPauseHtpPressed.png");
		};
		pauseHolder.addChild(htpBtn);

		replayBtn.loadBitmap("media/images/buttons/btnPauseReplay.png");
		replayBtn.x = 568;
		replayBtn.y = 672;
		replayBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnPauseReplay.png");
		};
		replayBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnPauseReplayPressed.png");
		};
		pauseHolder.addChild(replayBtn);

		quitBtn.loadBitmap("media/images/buttons/btnPauseQuit.png");
		quitBtn.x = 657;
		quitBtn.y = 672;
		quitBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnPauseQuit.png");
		};
		quitBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnPauseQuitPressed.png");
		};
		pauseHolder.addChild(quitBtn);

		htpHolder.loadBitmap("media/images/screens/htp/panel.png");
		htpHolder.x = 295;
		htpHolder.y = 18;
		htpHolder.alpha = 0;
		htpHolder.setIndex(3);
		overlayHolder.addChild(htpHolder);

		htpTitle.width = 420;
		htpTitle.height = 100;
		htpTitle.x = (579 - htpTitle.width) / 2;
		htpTitle.y = 78 - 18;
		htpTitle.font = overlayTitleFont;
		htpTitle.setStrokeText("HtpTitle");
		htpHolder.addChild(htpTitle);

		var htpText = new TextField("htpText");
		htpText.width = 420;
		htpText.height = 100;
		htpText.x = (579 - htpText.width) / 2;
		htpText.y = 183;
		htpText.font = overlayTextFont;
		htpText.setStrokeText("htp1"+device);
		htpHolder.addChild(htpText);

		htpImage.loadBitmap("media/images/screens/htp/htp1_" + device + ".png");
		htpImage.x = 346-295;
		htpImage.y = 284 -30;
		htpHolder.addChild(htpImage);

		htpPrev.loadBitmap("media/images/buttons/btnPrevBig.png");
		htpPrev.x = 111-295;
		htpPrev.y = 240 - 18;
		htpPrev.alpha = 0;;
		htpPrev.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnPrevBig.png");
		};
		htpPrev.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnPrevBigPressed.png");
		};
		htpHolder.addChild(htpPrev);

		htpNext.loadBitmap("media/images/buttons/btnNextBig.png");
		htpNext.x = 842-295;
		htpNext.y = 240 - 18;
		htpNext.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnNextBig.png");
		};
		htpNext.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnNextBigPressed.png");
		};
		htpHolder.addChild(htpNext);

		htpClose.loadBitmap("media/images/screens/chooseLevel/panelClose.png");
		htpClose.x = 769 - 295;
		htpClose.y = 64 - 18;
		htpClose.releasedState = function(){
			this.loadBitmap("media/images/screens/chooseLevel/panelClose.png");
		};
		htpClose.pressedState = function(){
			this.loadBitmap("media/images/screens/chooseLevel/panelClose.png");
		};
		htpHolder.addChild(htpClose);

		// Overlay

		quitHolder.loadBitmap("media/images/screens/confirm/panel.png");
		quitHolder.x = 273;
		quitHolder.y = 20;
		quitHolder.alpha = 0;
		quitHolder.setIndex(3);
		overlayHolder.addChild(quitHolder);

		quitTitle.width = 420;
		quitTitle.height = 100;
		quitTitle.x = (579 - quitTitle.width) / 2;
		quitTitle.y = 66;
		quitTitle.font = overlayTitleFont;
		quitTitle.setStrokeText("QuitTitle");
		quitHolder.addChild(quitTitle);

		quitText.width = 420;
		quitText.height = 100;
		quitText.x = (579 - quitText.width) / 2;
		quitText.y = 183;
		quitText.font = overlayTextFont;
		quitText.setStrokeText("QuitText");
		quitHolder.addChild(quitText);

		confirmQuitBtn.loadBitmap("media/images/buttons/btnSmall.png");
		confirmQuitBtn.x = 72;
		confirmQuitBtn.y = 268;
		confirmQuitBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnSmall.png");
		};
		confirmQuitBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnSmallPressed.png");
		};
		quitHolder.addChild(confirmQuitBtn);

		confirmQuitLabel.width = 169;
		confirmQuitLabel.height = 70;
		confirmQuitLabel.font = overlayBtnFont;
		confirmQuitLabel.setStrokeText("Yes");
		confirmQuitBtn.addChild(confirmQuitLabel);

		cancelQuitBtn.loadBitmap("media/images/buttons/btnSmall.png");
		cancelQuitBtn.x = 336;
		cancelQuitBtn.y = 268;
		cancelQuitBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnSmall.png");
		};
		cancelQuitBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnSmallPressed.png");
		};
		quitHolder.addChild(cancelQuitBtn);

		cancelQuitLabel.width = 169;
		cancelQuitLabel.height = 70;
		cancelQuitLabel.font = overlayBtnFont;
		cancelQuitLabel.setStrokeText("No");
		cancelQuitBtn.addChild(cancelQuitLabel);

		// Methods
		
		this.onload = function(){
			setTimeout(function(){ Tween.add(overlayHolder, { alpha: 1 }, 12, 0.5, function(){}); }, 50);
			setTimeout(function(){ Tween.add(resumeBtn, { y: 6 }, 12, 0.5, function(){ }); }, 100);
			setTimeout(function(){ Tween.add(muteBtn, { y: 442 }, 12, 0.5, function(){ }); }, 200);
			setTimeout(function(){ Tween.add(unmuteBtn, { y: 442 }, 12, 0.5, function(){ }); }, 200);
			setTimeout(function(){ Tween.add(htpBtn, { y: 442 }, 12, 0.5, function(){ }); }, 300);
			setTimeout(function(){ Tween.add(replayBtn, { y: 442 }, 12, 0.5, function(){ }); }, 300);
			setTimeout(function(){ Tween.add(quitBtn, { y: 442 }, 12, 0.5, function(){ }); }, 400);
		};	
		
		this.update = function(){}
		
		this.onunload = function(){};

		this.resume = function(){
			if(isOverlay){ return; }

			Tween.add(quitBtn, { alpha: 0, y: 672 }, 12, 0.5, function(){});
			Tween.add(replayBtn, { alpha: 0, y: 672 }, 12, 0.5, function(){});
			Tween.add(muteBtn, { alpha: 0, y: 672 }, 12, 0.5, function(){});
			Tween.add(unmuteBtn, { alpha: 0, y: 672 }, 12, 0.5, function(){});
			Tween.add(htpBtn, { alpha: 0, y: 672 }, 12, 0.5, function(){});
			Tween.add(resumeBtn, { alpha: 0, y: -507 }, 12, 0.5, function(){});
			Tween.add(overlayHolder, { alpha: 0 }, 12, 0.5, function(){
				EventManager.broadcast("hidePopup", "popupPause");	
			});
		};
		
		this.mute = function(){
        	if(isOverlay){ return; }

        	if(buttonEnabled){
				GameData.set("muted", "true", true);
				//console.log(GameData.get("sprite"));
				//console.log(GameData.get("music"));
				GameData.get("sprite").volume(0);
				for(var i=0; i<GameData.get("music").length; i++) GameData.get("music")[i].volume(0);
				buttonEnabled = false;
				muteBtn.alpha = 0;
				unmuteBtn.alpha = 1;

				setTimeout(function(){ buttonEnabled = true; }, 100);
			}
        }

        this.unmute = function(){
        	if(isOverlay){ return; }

        	if(buttonEnabled){
				GameData.set("muted", "false", true);
				GameData.get("sprite").volume(1);
				for(var i=0; i<GameData.get("music").length; i++) GameData.get("music")[i].volume(1);
				muteBtn.alpha = 1;
				unmuteBtn.alpha = 0;
				buttonEnabled = false;

				setTimeout(function(){ buttonEnabled = true; }, 100);
			}
        }

        this.replay = function(){
			EventManager.broadcast("gotoScreen", "gameplay");
		};

		this.showHtp = function()
		{
			if(isOverlay || isHtp){ return; }

			htpPrev.alpha = 0;
			htpNext.alpha = 1;

			htpCurrentPage = 1;

			pauseHolder.alpha = 0;
			htpHolder.alpha = 1;
			isHtp = true;
		};

		this.hideHtp = function()
		{
			pauseHolder.alpha = 1;
			htpHolder.alpha = 0;
			isHtp = false;
		};


        this.showOverlay = function(){
			if(isOverlay){ return; }

			pauseHolder.alpha = 0;
			quitHolder.alpha = 1;
			isOverlay = true;
		};

		this.hideOverlay = function(){
			pauseHolder.alpha = 1;
			quitHolder.alpha = 0;
			isOverlay = false;
		};

		this.quit = function(){
			EventManager.broadcast("gotoScreen", "title");
		};

		this.prevPage = function(){
			if(htpCurrentPage > 1){
				htpCurrentPage--;
			}

			thisRef.updatePage();
		};

		this.nextPage = function(){
			if(htpCurrentPage < htpTotalPage){
				htpCurrentPage++;
			}

			thisRef.updatePage();
		};

		this.updatePage = function(){
			switch(htpCurrentPage){
				case 1:
					htpPrev.alpha = 0;
					htpNext.alpha = 1;
					
					htpText.setStrokeText("htp"+htpCurrentPage+device);
					htpImage.loadBitmap("media/images/screens/htp/htp" + htpCurrentPage + '_' + device + ".png");
				break;
				default:
					if(htpCurrentPage == htpTotalPage){
						htpNext.alpha = 0;
					}
					else{
						htpNext.alpha = 1;
					}
					htpPrev.alpha = 1;
					htpText.setStrokeText("htp"+htpCurrentPage);
					htpImage.loadBitmap("media/images/screens/htp/htp" + htpCurrentPage + ".png");
				break;
			}
		};

		this.binds = {
			"resume": thisRef.resume,
			"mute": thisRef.mute,
            "unmute": thisRef.unmute,
			"replay": thisRef.replay,
			"quit": thisRef.showOverlay,
			"htp": thisRef.showHtp,
			"htpClose": thisRef.hideHtp,
			"prev": thisRef.prevPage,
			"next": thisRef.nextPage,
			"cancelQuit": thisRef.hideOverlay,
			"confirmQuit": thisRef.quit
		};
	};
	
	return screen;
});