define("screens/title", [
	"utils/Background",
	"sdk/EventManager",
	"sdk/DisplayObject",
	"sdk/TextField",
	"sdk/GameData",
	"sdk/Sound",
	"sdk/Tween",
	"text/Font"
], 
function (Background, EventManager, DisplayObject, TextField, GameData, Sound, Tween, Font){
	var screen = function(){
		var thisRef = this,
			// Local
			isOverlay = false,
			// Styles
			messagingFont = new Font("messagingFont"),
			playBtnFont = new Font("playBtnFont"),
			titleBtnFont = new Font("titleBtnFont"),
			overlayTitleFont = new Font("overlayTitleFont"),
			overlayTextFont = new Font("overlayTextFont"),
			overlayBtnFont = new Font("overlayBtnFont"),			
			// Elements
			badge = new DisplayObject("badge"),
			property = new DisplayObject("property"),
			messagingText = new TextField("messagingText"),
			logo = new DisplayObject("logo"),
			playBtn = new DisplayObject("play"),
				playLabel = new TextField("playLabel"),
			resetBtn = new DisplayObject("reset"),
				resetLabel = new TextField("resetLabel"),
			continueBtn = new DisplayObject("continue"),
				continueLabel = new TextField("continueLabel"),
			copyright = new DisplayObject("copyright"),
			overlayHolder = new DisplayObject("overlayHolder"),
			eraseHolder = new DisplayObject("eraseHolder"),
				overlayTitle = new TextField("overlayTitle"),
				overlayText = new TextField("overlayText"),
				confirmResetBtn = new DisplayObject("confirmReset"),
					confirmResetLabel = new TextField("confirmResetLabel"),
				cancelResetBtn = new DisplayObject("cancelReset"),
					cancelResetLabel = new TextField("cancelResetLabel")
		;
		
		Background.set("media/images/screens/title/bg.jpg");

		// Fonts

		messagingFont.font = "18px NexaBold";
		messagingFont.fillStyle = "#FFF";
		messagingFont.textBaseline = "alphabetical";
		messagingFont.paddingTop = 22;
		messagingFont.textAlign = "center";
		messagingFont.textWrap = true;

		playBtnFont.font = "28px NexaBold";
		playBtnFont.fillStyle = "#FFF";
		playBtnFont.textBaseline = "middle";
		playBtnFont.paddingTop = 38; // PaddingTop equal to 50% of the container height (less 4px because of button bottom shadow)
		playBtnFont.textAlign = "center";

		titleBtnFont.font = "18px NexaBold";
		titleBtnFont.fillStyle = "#FFF";
		titleBtnFont.textBaseline = "middle";
		titleBtnFont.paddingTop = 38; // PaddingTop equal to 50% of the container height (less 4px because of button bottom shadow)
		titleBtnFont.textAlign = "center";

		overlayTitleFont.font = "56px NexaBold";
		overlayTitleFont.fillStyle = "#60D5FF";
		// Firefox display textBaseline = top differently that in other browser.
		// To fix it, we can simply set textBaseline = alphabetical and set a paddingTop equal to the font size. 
		overlayTitleFont.textBaseline = "alphabetical";
		overlayTitleFont.paddingTop = 56;
		overlayTitleFont.textAlign = "center";

		overlayTextFont.font = "25px NexaBold";
		overlayTextFont.fillStyle = "#60D5FF";
		// Firefox display textBaseline = top differently that in other browser.
		// To fix it, we can simply set textBaseline = alphabetical and set a paddingTop equal to the font size. 
		overlayTextFont.textBaseline = "alphabetical";
		overlayTextFont.paddingTop = 25;
		overlayTextFont.textAlign = "center";
		overlayTextFont.verticalSpacing = 5;
		overlayTextFont.textWrap = true;

		overlayBtnFont.font = "25px NexaBold";
		overlayBtnFont.fillStyle = "#FFF";
		overlayBtnFont.textBaseline = "middle";
		overlayBtnFont.paddingTop = 38; // PaddingTop equal to 50% of the container height (less 4px because of button bottom shadow)
		overlayBtnFont.textAlign = "center";

		/* This fix the 4px font gap in Firefox (might be integrate later right in the SDK) */
		if(GameContainer.browserInfo.browserName === "Firefox"){
			titleBtnFont.paddingTop -= 2;

			/*
			badgeTextFont.paddingTop += 4;
			overlayBtnFont.paddingTop += 4;
			*/
		}
		
		// Default
		property.loadBitmap("media/images/localized/property.png");
		property.x = 661;
		property.y = 46;
		this.addChild(property);

		messagingText.width = 300;
		messagingText.height = 100;
		messagingText.x = 800 - (messagingText.width / 2);
		messagingText.y = 236;
		messagingText.font = messagingFont;
		messagingText.setFillText("TitleMessaging");
		if(GameData.get("messaging") == true){
			this.addChild(messagingText);
		}

		logo.loadBitmap("media/images/localized/logo.png");
		logo.x = 660;
		logo.y = 286;
		this.addChild(logo);

		playBtn.loadBitmap("media/images/buttons/btnTitle.png");
		playBtn.x = 671;
		playBtn.y = 672; // 483;
		playBtn.alpha = 0;
		playBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnTitle.png");
		};
		playBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnTitlePressed.png");
		};
		this.addChild(playBtn);
		
		playLabel.width = 213;
		playLabel.height = 68;
		playLabel.font = playBtnFont;
		playLabel.setFillText("Play");
		playBtn.addChild(playLabel);

		resetBtn.loadBitmap("media/images/buttons/btnTitle.png");
		resetBtn.x = 570;
		resetBtn.y = 672; // 483;
		resetBtn.alpha = 0;
		resetBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnTitle.png");
		};
		resetBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnTitlePressed.png");
		};
		this.addChild(resetBtn);

		resetLabel.width = 213;
		resetLabel.height = 68;
		resetLabel.font = titleBtnFont;
		resetLabel.setFillText("NewGame");
		resetBtn.addChild(resetLabel);

		continueBtn.loadBitmap("media/images/buttons/btnTitle.png");
		continueBtn.x = 806;
		continueBtn.y = 672; // 483;
		continueBtn.alpha = 0;
		continueBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnTitle.png");
		};
		continueBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnTitlePressed.png");
		};
		this.addChild(continueBtn);

		continueLabel.width = 213;
		continueLabel.height = 68;
		continueLabel.font = titleBtnFont;
		continueLabel.setFillText("Continue");
		continueBtn.addChild(continueLabel);

		copyright.loadBitmap("media/images/localized/copyright.png");
		copyright.x = 90;
		copyright.y = 565;
		this.addChild(copyright);

		// Overlay

		overlayHolder.loadBitmap("media/images/backgrounds/overlayBg.png");
		overlayHolder.alpha = 0;
		overlayHolder.setIndex(2);
		this.addChild(overlayHolder);

		eraseHolder.loadBitmap("media/images/screens/confirm/panel.png");
		eraseHolder.x = 292;
		eraseHolder.y = -643; // 20
		overlayHolder.addChild(eraseHolder);

		overlayTitle.width = 420;
		overlayTitle.height = 100;
		overlayTitle.x = (579 - overlayTitle.width) / 2;
		overlayTitle.y = 66;
		overlayTitle.font = overlayTitleFont;
		overlayTitle.setStrokeText("ResetTitle");
		eraseHolder.addChild(overlayTitle);

		overlayText.width = 420;
		overlayText.height = 100;
		overlayText.x = (579 - overlayText.width) / 2;
		overlayText.y = 183;
		overlayText.font = overlayTextFont;
		overlayText.setStrokeText("ResetText");
		eraseHolder.addChild(overlayText);

		confirmResetBtn.loadBitmap("media/images/buttons/btnSmall.png");
		confirmResetBtn.x = 72;
		confirmResetBtn.y = 300;
		confirmResetBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnSmall.png");
		};
		confirmResetBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnSmallPressed.png");
		};
		eraseHolder.addChild(confirmResetBtn);

		confirmResetLabel.width = 169;
		confirmResetLabel.height = 70;
		confirmResetLabel.font = overlayBtnFont;
		confirmResetLabel.setStrokeText("Yes");
		confirmResetBtn.addChild(confirmResetLabel);

		cancelResetBtn.loadBitmap("media/images/buttons/btnSmall.png");
		cancelResetBtn.x = 336;
		cancelResetBtn.y = 300;
		cancelResetBtn.releasedState = function(){
			this.loadBitmap("media/images/buttons/btnSmall.png");
		};
		cancelResetBtn.pressedState = function(){
			this.loadBitmap("media/images/buttons/btnSmallPressed.png");
		};
		eraseHolder.addChild(cancelResetBtn);

		cancelResetLabel.width = 169;
		cancelResetLabel.height = 70;
		cancelResetLabel.font = overlayBtnFont;
		cancelResetLabel.setStrokeText("No");
		cancelResetBtn.addChild(cancelResetLabel);
		
		

		// Sound
		// is window.bgMusic a safe/good way to hold a reference across screens?
		
		
			
			if(window.GameContainer.browserInfo.touchDevice){
				
				window.bgMusic = new Howl({
					urls:[
						'media/music/menu.mp3',
						'media/music/menu.ogg'
					],
					loop:true
				});
				
				function startMusic(){
					document.removeEventListener('touchstart', startMusic);
					console.log("PLAYING BG MUSIC...");
					window.bgMusic.play();
				}
				
				document.addEventListener('touchstart', startMusic);
			} else {
				window.bgMusic = new Howl({
					urls:[
						'media/music/menu.mp3',
						'media/music/menu.ogg'
					],
					autoplay:true,
					loop:true
				});
			}
			if(GameData.get("muted", true) == "true"){
				window.bgMusic.volume(0);
			}
			
		// Methods
		
		this.onload = function(){
			if(GameData.get("firstTime", true) == undefined){
				thisRef.setDefaultLocalStorage();

				setTimeout(function(){ Tween.add(playBtn, { alpha: 1, y: 510 }, 12, 0.5, function(){ }); }, 500);
			}
			else{
				setTimeout(function(){ Tween.add(resetBtn, { alpha: 1, y: 510 }, 8, 0.7, function(){ }); }, 500);
				setTimeout(function(){ Tween.add(continueBtn, { alpha: 1, y: 510 }, 8, 0.7, function(){ }); }, 600);
			}
		};	
		
		this.update = function(){};
		
		this.onunload = function(){};
		
		this.play = function(){
			if(isOverlay) return;

			GameData.set("firstTime", "true", true);

			EventManager.broadcast("gotoScreen", "story");
		};

		this.continue = function(){
			if(isOverlay) return;

			EventManager.broadcast("gotoScreen", "story");
		};

		this.showOverlay = function(){
			if(isOverlay) return;

			setTimeout(function(){ Tween.add(overlayHolder, { alpha: 1 }, 12, 0.5, function(){
				setTimeout(function(){ Tween.add(eraseHolder, { y: 20 }, 12, 0.5, function(){ isOverlay = true; }); }, 100);

			}); }, 50);
		};

		this.hideOverlay = function(){
			setTimeout(function(){ Tween.add(eraseHolder, { y: -643 }, 12, 0.5, function(){
				setTimeout(function(){ Tween.add(overlayHolder, { alpha: 0 }, 12, 0.5, function(){ isOverlay = false; }); }, 50);

			}); }, 100);
		};

		this.reset = function(){
			thisRef.setDefaultLocalStorage();

			EventManager.broadcast("gotoScreen", "story");
		};

		this.setDefaultLocalStorage = function(){
			GameData.set("firstTime", "false", true);
			GameData.set("tutDone", "false", true);
			GameData.set("muted", "false", true);

			GameData.set("showTutorial", "true", true);
			GameData.set("goToEnd", true, true);

			GameData.set("levelUnlocked", 4, true);

			GameData.set("level1Highscore", 0, true);
			GameData.set("level1Star", 0, true);
			GameData.set("level1Note", 0, true);
			GameData.set("level1Token1", "false", true);
			GameData.set("level1Token2", "false", true);
			GameData.set("level1Token3", "false", true);
			GameData.set("level1Token4", "false", true);
			GameData.set("level1Token5", "false", true);
			
			GameData.set("level2Highscore", 0, true);
			GameData.set("level2Star", 0, true);
			GameData.set("level2Note", 0, true);
			GameData.set("level2Token1", "false", true);
			GameData.set("level2Token2", "false", true);
			GameData.set("level2Token3", "false", true);
			GameData.set("level2Token4", "false", true);
			GameData.set("level2Token5", "false", true);
			
			GameData.set("level3Highscore", 0, true);
			GameData.set("level3Star", 0, true);
			GameData.set("level3Note", 0, true);
			GameData.set("level3Token1", "false", true);
			GameData.set("level3Token2", "false", true);
			GameData.set("level3Token3", "false", true);
			GameData.set("level3Token4", "false", true);
			GameData.set("level3Token5", "false", true);

			GameData.set("level4Highscore", 0, true);
			GameData.set("level4Star", 0, true);
			GameData.set("level4Note", 0, true);
			GameData.set("level4Token1", "false", true);
			GameData.set("level4Token2", "false", true);
			GameData.set("level4Token3", "false", true);
			GameData.set("level4Token4", "false", true);
			GameData.set("level4Token5", "false", true);

			GameData.set("level5Highscore", 0, true);
			GameData.set("level5Star", 0, true);
			GameData.set("level5Note", 0, true);
			GameData.set("level5Token1", "false", true);
			GameData.set("level5Token2", "false", true);
			GameData.set("level5Token3", "false", true);
			GameData.set("level5Token4", "false", true);
			GameData.set("level5Token5", "false", true);

			GameData.set("level6Highscore", 0, true);
			GameData.set("level6Star", 0, true);
			GameData.set("level6Note", 0, true);
			GameData.set("level6Token1", "false", true);
			GameData.set("level6Token2", "false", true);
			GameData.set("level6Token3", "false", true);
			GameData.set("level6Token4", "false", true);
			GameData.set("level6Token5", "false", true);

			GameData.set("artistsToGet0", JSON.stringify(thisRef.shuffle([0])), true);
			GameData.set("artistsToGet1", JSON.stringify(thisRef.shuffle([1,2,3,4,5])), true);
			GameData.set("artistsToGet2", JSON.stringify(thisRef.shuffle([1,2,3,4,5])), true);
			GameData.set("artistsToGet3", JSON.stringify(thisRef.shuffle([1,2,3,4,5])), true);
			GameData.set("artistsToGet4", JSON.stringify(thisRef.shuffle([1,2,3,4,5])), true);
			GameData.set("artistsToGet5", JSON.stringify(thisRef.shuffle([1,2,3,4,5])), true);
			GameData.set("artistsToGet6", JSON.stringify(thisRef.shuffle([1,2,3,4,5])), true);

		};

		this.shuffle = function(o){
		    for(var j, x, i = o.length; i; j = Math.floor(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
		    return o;
		};
		
		this.binds = {
			"play": thisRef.play,
			"continue": thisRef.continue,
			"reset": thisRef.showOverlay,
			"cancelReset": thisRef.hideOverlay,
			"confirmReset": thisRef.reset
		};
	};
	
	return screen;
});